#include "EXTERN.h"
#include "perl.h"
#include "XSUB.h"

#include "ppport.h"


enum ship_type {
    MIS, LAS, PUL, PAR, DEC, GAU, LAR, PLA,
    HRM, HEL, ART, ATL, APO, CHA, HRC, DIO,
    POS, GAI, ATH, ARE, HAD, PRO, ZEU, HEP,
};

static int rapidfire[24][24] = {
    /* Mis 0 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
    /* Las 1 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
    /* Pul 2 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
    /* Par 3 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
    /* Dec 4 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
    /* Gau 5 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
    /* Lar 6 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
    /* Pla 7 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
    /* Hrm 8 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
    /* Hel 9 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
    /* Art 10 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 5, 5, 0, 0, 0, 5, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
    /* Atl 11 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 5, 5, 0, 0, 0, 5, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
    /* Apo 12 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 5, 5, 0, 3, 0, 5, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
    /* Cha 13 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
    /* Hrc 14 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 5, 5, 0, 0, 0, 5, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
    /* Dio 15 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 5, 5, 0, 0, 0, 5, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
    /* Pos 16 */
    { 10, 0, 0, 0, 0, 0, 0, 0, 5, 5, 6, 0, 0, 5, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
    /* Gai 17 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 5, 5, 0, 3, 0, 5, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
    /* Ath 18 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 5, 5, 0, 0, 0, 5, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
    /* Are 19 */
    { 20, 20, 10, 10, 0, 0, 0, 0,
            5, 5, 0, 0, 0, 5, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
    /* Had 20 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 5, 5, 0, 3, 4, 5, 3, 0, 4, 0, 7, 0, 0, 0, 0, 0 },
    /* Pro 21 */
    { 0, 10, 0, 0, 0, 0, 0, 0, 5, 5, 0, 0, 0, 5, 0, 0, 0, 0, 0, 0, 2, 0, 0, 0 },
    /* Zeu 22 */
    { 200, 200, 100, 100, 0, 50, 0, 0,
            1250, 1250, 200, 250, 100, 1250, 250, 250,
            33, 250, 30, 25, 15, 5, 0, 0 },
    /* Hep 23 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 }
};

int st_index(const char *x) {
    switch (*x) {
    case 'A':
        switch (*(x+1)) {
        case 'p': return APO;
        case 'r':
            switch (*(x+2)) {
            case 'e': return ARE;
            case 't': return ART;
            }
        case 't':
            switch (*(x+2)) {
            case 'h': return ATH;
            case 'l': return ATL;
            }
        }
    case 'C': return(CHA);
    case 'D':
        switch (*(x+1)) {
        case 'e': return DEC;
        case 'i': return DIO;
        }
    case 'G':
        switch (*(x+2)) {
        case 'i': return GAI;
        case 'u': return GAU;
        }
    case 'H':
        switch (*(x+1)) {
        case 'a': return HAD;
        case 'e':
            switch (*(x+2)) {
            case 'l': return HEL;
            case 'p': return HEP;
            }
        case 'r':
            switch (*(x+2)) {
            case 'c': return HRC;
            case 'm': return HRM;
            }
        }
    case 'L':
        switch (*(x+2)) {
        case 'r': return LAR;
        case 's': return LAS;
        }
    case 'M': return(MIS);
    case 'P':
        switch (*(x+1)) {
        case 'a': return PAR;
        case 'l': return PLA;
        case 'o': return POS;
        case 'r': return PRO;
        case 'u': return PUL;
        }
    case 'Z': return(ZEU);
    }
}



MODULE = SFC::Rapidfire		PACKAGE = SFC::Rapidfire		

int
rapidfire_value(att, def)
        char *  att
        char *  def
    CODE:
        RETVAL = rapidfire[st_index(att)][st_index(def)];
    OUTPUT:
        RETVAL
