
package SFC::Fleet;

use strict;
use warnings qw{ all };

use Getopt::Long;
use SFC::Ship;
use SFC::ShipStats qw{ stdsort desc_stats };


sub summary {
    my $self = shift;

    my $summary = SFC::FleetSummary->summarize($self);
}


sub summary1 {
    my $self = shift;

    my %summary;

    for my $s (@{$self->{ships}}) {
        $summary{$s->type()} += $s;
    }
    
    return \%summary;
}


sub summary_str {
    my $self = shift;

    my $summary = $self->summary1();
    my @sumstr;

    for my $t (sort stdsort keys %$summary) {
        push @sumstr, $summary->{$t} . '';
        # push @sumstr, $summary->{$t}->ship_str();
    }

    return @sumstr;
}


sub attack {
    my $self = shift;
    my ($opponent) = @_;

    for my $s (@{$self->{ships}}) {
        $s->attack($opponent, $self);
    }
}

sub nostat_attack {
    my $self = shift;
    my ($opponent) = @_;

    for my $s (@{$self->{ships}}) {
        $s->nostat_attack($opponent, $self);
    }
}


sub explosion {
    my $self = shift;

    my @remains;
    my $res = $self->{results};

    for my $s (@{$self->{ships}}) {
        if ($s->explodes()) {
            $res->[$s->id()]->ship_lost($s);
        }
        else {
            push @remains, $s;
        }
    }

    $self->{ships} = \@remains;
}

sub reset_shields {
    my $self = shift;

    for my $s (@{$self->{ships}}) {
        $s->reset_shield();
    }
}

sub reset_detonations {  # how many of the opponent's mines can detonate
    my $self = shift;

    $self->{detonations} = 5 * @{$self->{ships}};
}

sub print_status {
    my $self = shift;

    for my $s (@{$self->{ships}}) {
        $s->print_status();
    }
}

sub destroyed {
    my $self = shift;

    return ($#{$self->{ships}} < 0);
}


sub new {
    my $this = shift;
    my $class = ref($this) || $this;
    my $self = ref($this) ? { %$this } : {
            # defaults
            results => [],
        };
    bless $self, $class;

    while (@_) {
        my $arg = shift;
        if (ref($arg) eq 'SFC::FleetSummary') {
            $self->add_summary($arg);
        }
        elsif (ref($arg)) {
            while (my ($k, $v) = each %$arg) { $self->{$k} = $v; }
        }
        else {
            $self->{$arg} = shift;
        }
    }

    return $self;
}


sub add_ships {
    my $self = shift;
    my ($type, $num, $id) = @_;

    for (; $num; --$num) {
        push @{$self->{ships}},
                SFC::Ship->new($type, $self->{results}[$id], $id,
                            $self->{is_attacker});
    }
}

sub clone {
    my $this = shift;
    my $class = ref($this) || $this;
    my $self = ref($this) ? { %$this } : { };
    bless $self, $class;

    $self->{ships} = [];
    for my $s (@{$this->{ships}}) {
        push @{$self->{ships}}, $s->new();
    }

    return $self;
}

sub add_summary {
    my $self = shift;
    my ($summary) = @_;
    my $id = @{$self->{results}} || 0;

    my $result = $summary->new();
    push @{$self->{results}}, $result;

    for my $t (keys %$summary) {
        next unless $t =~ /^[A-Z][a-z][a-z]$/;
        my $num = $summary->{$t}{count};
        next unless $num > 0;
        $result->{$t} = {};
        for my $k (desc_stats) {
            $result->{$t}{$k} = 0;
        }
        $result->{$t}{count} = $num;
        $result->{$t}{gen_loss} = $summary->{$t}{gen_loss};
        $result->{$t}{loss} = $summary->{$t}{loss};

        my $proto = SFC::Ship->new($t, $summary, $id, $self->{is_attacker});
        push @{$self->{ships}}, $proto;
        --$num;
        for (; $num; --$num) {
            push @{$self->{ships}}, $proto->new();
        }
    }

    return $self;
}

sub results {
    my $self = shift;
    my ($arg) = @_;

    if (!defined $arg) {
        return $self->{results};
    }
    if (ref($arg)) {
        return $self->{results}[$arg->id()];
    }
    return $self->{results}[$arg];
}

sub ships { $_[0]->{ships} }
sub weapon_tech { $_[0]->{weapon} }
sub shield_tech { $_[0]->{shield} }
sub armor_tech { $_[0]->{armor} }
sub ore_lost { $_[0]->{ore} }
sub crystal_lost { $_[0]->{cry} }
sub hydrogen_lost { $_[0]->{gen} }
sub debris { $_[0]->{debris} }
sub detonations_left { $_[0]->{detonations} }
sub detonation { --$_[0]->{detonations} }

1;
