
package SFC::ShipStats;

use strict;
use warnings qw{ all };



our (@ISA, @EXPORT_OK);
BEGIN {
    require Exporter;
    @ISA = qw{Exporter};
    @EXPORT_OK = qw{
            full_name
            ore_cost
            crystal_cost
            hydrogen_cost
            fuel_consumption
            is_ship
            sort_order
            drop_order
            cargo_space
            attack_strength
            shield_strength
            hull
            base_speed
            engine
            speed
            stdsort
            drop_sort
            rapidfire_value
            desc_stats
            all_units
            base_hull
            base_weapon
            base_shield
        };
}

sub desc_stats {
    qw{
        count ships_lost
    };
#        overkill_given overkill_taken
#        ir_loss ir_gain
#        ore_loss cry_loss gen_loss loss
#        effect
}                                                             


my %base_stats = (
        Sha => {
                fullname => 'Shadow Probe',
                ship => 1,
                ore => 0,
                cry => 0,
                gen => 0,
                cargo => 5,
                engine => 'jet',
                speed => 100000000,
                fuel => 1,
                weapon => 0,
                shield => 0,
                hull => 100,
                sorto => 1,
            },
        Gen => {
                fullname => 'Genesis Class Solar Satellite',
                ship => 0,
                ore => 0,
                cry => 0,
                gen => 0,
                weapon => 1,
                shield => 1,
                hull => 200,
                sorto => 2,
            },
        Hrm => {
                fullname => 'Hermes Class Probe',
                ship => 1,
                ore => 0,
                cry => 1000,
                gen => 0,
                cargo => 5,
                engine => 'jet',
                speed => 100000000,
                fuel => 1,
                weapon => 0,
                shield => 0,
                hull => 100,
                sorto => 3,
            },
        Hel => {
                fullname => 'Helios Class Solar Satellite',
                ship => 1,
                ore => 0,
                cry => 2000,
                gen => 500,
                cargo => 0,
                engine => 'jet',
                speed => 0,
                fuel => 0,
                weapon => 1,
                shield => 1,
                hull => 200,
                sorto => 4,
            },
        Art => {
                fullname => 'Artemis Class Fighter',
                ship => 1,
                ore => 3000,
                cry => 1000,
                gen => 0,
                cargo => 50,
                engine => 'jet',
                speed => 12500,
                fuel => 20,
                weapon => 50,
                shield => 10,
                hull => 400,
                sorto => 5,
            },
        Atl => {
                fullname => 'Atlas Class Cargo',
                ship => 1,
                ore => 2000,
                cry => 2000,
                gen => 0,
                cargo => 5000,
                engine => 'pulse',  # upgraded from jet
                speed => 10000,     # upgraded from 5000
                fuel => 20,         # "upgraded" from 10
                weapon => 5,
                shield => 10,
                hull => 400,
                sorto => 6,
            },
        Apo => {
                fullname => 'Apollo Class Fighter',
                ship => 1,
                ore => 6000,
                cry => 2500,
                gen => 0,
                cargo => 100,
                engine => 'pulse',
                speed => 10000,
                fuel => 75,
                weapon => 150,
                shield => 25,
                hull => 850,
                sorto => 7,
            },
        Zag => {
                fullname => 'Zagreus Class Recycler',
                ship => 1,
                ore => 5000,
                cry => 3000,
                gen => 1000,
                cargo => 5000,
                engine => 'jet',
                speed => 3500,
                fuel => 400,
                weapon => 1,
                shield => 5,
                hull => 800,
                sorto => 8,
            },
        Cha => {
                fullname => 'Charon Class Transport',
                ship => 1,
                ore => 4000,
                cry => 4000,
                gen => 1000,
                cargo => 100,
                engine => 'pulse',
                speed => 10000,
                fuel => 20,
                weapon => 1,
                shield => 25,
                hull => 800,
                sorto => 9,
            },
        Hrc => {
                fullname => 'Hercules Class Cargo',
                ship => 1,
                ore => 6000,
                cry => 6000,
                gen => 0,
                cargo => 25000,
                engine => 'jet',
                speed => 7500,
                fuel => 50,
                weapon => 5,
                shield => 25,
                hull => 1200,
                sorto => 10,
            },
        Emp => {
                fullname => 'Empusa Class Fighter',
                ship => 1,
                ore => 500,
                cry => 0,
                gen => 7500,
                cargo => 25,
                engine => 'jet',
                speed => 17500,
                fuel => 10,
                weapon => 1250,
                shield => 1,
                hull => 50,
                sorto => 11,
            },
        Dio => {
                fullname => 'Dionysus Class Recycler',
                ship => 1,
                ore => 10000,
                cry => 6000,
                gen => 2000,
                cargo => 20000,
                engine => 'jet',
                speed => 2000,
                fuel => 300,
                weapon => 1,
                shield => 10,
                hull => 1600,
                sorto => 12,
            },
        Cur => {
                fullname => 'Curetes Class Cruiser',
                ship => 1,
                ore => 2000,
                cry => 2000,
                gen => 10000,
                cargo => 250,
                engine => 'jet',
                speed => 12000,
                fuel => 25,
                weapon => 100,
                shield => 1800,
                hull => 400,
                sorto => 13,
            },
        Pos => {
                fullname => 'Posidon Class Cruiser',
                ship => 1,
                ore => 20000,
                cry => 7000,
                gen => 2000,
                cargo => 800,
                engine => 'pulse',
                speed => 15000,
                fuel => 300,
                weapon => 400,
                shield => 50,
                hull => 2700,
                sorto => 14,
            },
        Car => {
                fullname => 'Carmanor Class Cargo',
                ship => 1,
                ore => 18000,
                cry => 18000,
                gen => 0,
                cargo => 125000,
                engine => 'jet',
                speed => 4000,
                fuel => 120,
                weapon => 10,
                shield => 100,
                hull => 3600,
                sorto => 15,
            },
        Pal => {
                fullname => 'Pallas Class Bomber',
                ship => 1,
                ore => 25000,
                cry => 12500,
                gen => 7500,
                cargo => 250,
                engine => 'pulse',
                speed => 6000,
                fuel => 1000,
                weapon => 500,
                shield => 250,
                hull => 3750,
                sorto => 16,
            },
        Gai => {
                fullname => 'Gaia Class Colony Ship',
                ship => 1,
                ore => 10000,
                cry => 20000,
                gen => 10000,
                cargo => 7500,
                engine => 'pulse',
                speed => 2500,
                fuel => 1000,
                weapon => 50,
                shield => 100,
                hull => 3000,
                sorto => 17,
            },
        Ath => {
                fullname => 'Athena Class Battleship',
                ship => 1,
                ore => 45000,
                cry => 15000,
                gen => 0,
                cargo => 1500,
                engine => 'warp',
                speed => 10000,
                fuel => 500,
                weapon => 1000,
                shield => 200,
                hull => 6000,
                sorto => 18,
            },
        Are => {
                fullname => 'Ares Class Bomber',
                ship => 1,
                ore => 50000,
                cry => 25000,
                gen => 15000,
                cargo => 500,
                engine => 'warp',   # upgraded from pulse
                speed => 5000,      # upgraded from 4000
                fuel => 1000,
                weapon => 1000,
                shield => 500,
                hull => 7500,
                sorto => 19,
            },
        Had => {
                fullname => 'Hades Class Battleship',
                ship => 1,
                ore => 30000,
                cry => 40000,
                gen => 15000,
                cargo => 750,
                engine => 'warp',
                speed => 10000,
                fuel => 250,
                weapon => 700,
                shield => 400,
                hull => 7000,
                sorto => 20,
            },
        Pro => {
                fullname => 'Prometheus Class Destroyer',
                ship => 1,
                ore => 60000,
                cry => 50000,
                gen => 15000,
                cargo => 2000,
                engine => 'warp',
                speed => 5000,
                fuel => 1000,
                weapon => 2000,
                shield => 500,
                hull => 11000,
                sorto => 21,
            },
        Tha => {
                fullname => 'Thanatos Class Destroyer',
                ship => 1,
                ore => 1750000,
                cry => 800000,
                gen => 100000,
                cargo => 200000,
                engine => 'warp',
                speed => 1000,
                fuel => 750,
                weapon => 20000,
                shield => 10000,
                hull => 255000,
                sorto => 22,
            },
        Zeu => {
                fullname => 'Zeus Class',
                ship => 1,
                ore => 5000000,
                cry => 4000000,
                gen => 1000000,
                cargo => 1000000,
                engine => 'warp',
                speed => 100,
                fuel => 1,
                weapon => 200000,
                shield => 50000,
                hull =>   900000,
                sorto => 23,
            },
        Hep => {
                fullname => 'Hephaestus Class Attack Platform',
                ship => 1,
                ore => 20000000,
                cry => 20000000,
                gen => 10000000,
                cargo => 1000000000,
                engine => 'warp',
                speed => 90,
                fuel => 450000,
                weapon => 0,
                shield => 150000,
                hull   => 4000000,
                sorto => 24,
            },
        Mis => {
                fullname => 'Missile Battery',
                ship => 0,
                ore => 2000,
                cry => 0,
                gen => 0,
                weapon => 80,
                shield => 20,
                hull => 200,
                sorto => 31,
            },
        Las => {
                fullname => 'Laser Cannon',
                ship => 0,
                ore => 1500,
                cry => 500,
                gen => 0,
                weapon => 100,
                shield => 25,
                hull => 200,
                sorto => 32,
            },
        Min => {
                fullname => 'Space Mine',
                ship => 0,
                ore => 500,
                cry => 0,
                gen => 3000,
                weapon => 1000,
                shield => 1,
                hull   => 50,
                sorto => 33,
            },
        Pul => {
                fullname => 'Pulse Cannon',
                ship => 0,
                ore => 6000,
                cry => 2000,
                gen => 0,
                weapon => 250,
                shield => 100,
                hull => 800,
                sorto => 34,
            },
        Par => {
                fullname => 'Particle Cannon',
                ship => 0,
                ore => 2000,
                cry => 6000,
                gen => 0,
                weapon => 150,
                shield => 500,
                hull => 800,
                sorto => 35,
            },
        Dec => {
                fullname => 'Decoy',
                ship => 0,
                ore => 10000,
                cry => 10000,
                gen => 0,
                weapon => 0,
                shield => 2000,
                hull => 2000,
                sorto => 36,
            },
        Gau => {
                fullname => 'Gauss Cannon',
                ship => 0,
                ore => 20000,
                cry => 15000,
                gen => 2000,
                weapon => 1100,
                shield => 200,
                hull => 3500,
                sorto => 37,
            },
        Lar => {
                fullname => 'Large Decoy',
                ship => 0,
                ore => 50000,
                cry => 50000,
                gen => 0,
                weapon => 0,
                shield => 10000,
                hull => 10000,
                sorto => 38,
            },
        Pla => {
                fullname => 'Plasma Cannon',
                ship => 0,
                ore => 50000,
                cry => 50000,
                gen => 30000,
                weapon => 3000,
                shield => 300,
                hull => 10000,
                sorto => 39,
            },
    );

{
    my $i = 1;
    for my $s (
                'Sha', 'Gen', 'Hel',
                'Zag', 'Dio', 'Cha', 'Gai', 'Emp',
                'Atl', 'Hrc', 'Car',
                'Hrm', 'Art',
                'Apo', 'Pos', 'Pal',
                'Ath', 'Are', 'Had', 'Pro',
                'Cur', 'Tha', 'Zeu', 'Hep',
                'Mis', 'Las', 'Min', 'Pul', 'Par',
                'Dec', 'Gau', 'Lar', 'Pla', 
                ) {
        $base_stats{$s}{dropo} = $i;
        ++$i;
    }
}


my %rapidfire = (
        Art => { Hel => 5, Gen => 5, Hrm => 5, Cha => 5, },
        Apo => { Hel => 5, Gen => 5, Hrm => 5, Atl => 3, Cha => 5, },
        Atl => { Hel => 5, Gen => 5, Hrm => 5, Cha => 5, },
        Dio => { Hel => 5, Gen => 5, Hrm => 5, Cha => 5, },
        Gai => { Hel => 5, Gen => 5, Hrm => 5, Cha => 5, },
        Hrc => { Hel => 5, Gen => 5, Hrm => 5, Cha => 5, },
        Cur => { Hel => 5, Gen => 5, Hrm => 5, Cha => 5, },
        Pal => { Hel => 5, Gen => 5, Hrm => 5, Cha => 5, 
                    Mis => 50, Las => 50, Min => 100, },
        Pos => { Art => 6, Emp => 3, Cha => 5, Hel => 5, Gen => 5, Hrm => 5,
                    Mis => 10, },
        Ath => { Hel => 5, Gen => 5, Hrm => 5, Cha => 5, },
        Are => { Hel => 5, Gen => 5, Hrm => 5, Cha => 5, 
                    Mis => 20, Las => 20, Pul => 10, Par => 10, },
        Had => { Hel => 5, Gen => 5, Hrm => 5, Apo => 4, Atl => 3, Cha => 5,
                    Hrc => 3, Pos => 4, Ath => 7, },
        Pro => { Hel => 5, Gen => 5, Hrm => 5, Cha => 5, Had => 2, Las => 10, },
        Tha => { Hel => 250, Gen => 250, Hrm => 250, Cha => 250,
                    Art => 40, Emp => 30, Apo => 20,
                    Zag => 100, Dio => 50, Gai => 50, Pal => 100,
                    Atl => 50, Hrc => 50, Car => 25, 
                    Pos => 7, Ath => 6, Are => 5, Had => 3, Pro => 2,
                    Mis => 40, Las => 40, Pul => 20, Par => 20, Gau => 10, },
        Zeu => { Hel => 1250, Gen => 1250, Hrm => 1250, Cha => 1250,
                    Art => 200, Emp => 150, Apo => 100, Min => 75,
                    Zag => 500, Dio => 250, Gai => 250, Pal => 500,
                    Atl => 250, Hrc => 250, Car => 125, Tha => 2,
                    Pos => 33, Ath => 30, Are => 25, Had => 15, Pro => 5,
                    Mis => 200, Las => 200, Pul => 100, Par => 100, Gau => 50,
                },
    );

sub rapidfire_value {
    # my ($att, $def) = @_;
    # return $rapidfire{$att}{$def} || 0;
    return $rapidfire{$_[0]}{$_[1]};
}

sub attack_strength {
    my ($type, $fs) = @_;

    return int($base_stats{$type}{weapon} * (1 + 0.1 * $fs->weapon_tech));
}


sub shield_strength {
    my ($type, $fs) = @_;

    return int($base_stats{$type}{shield} * (1 + 0.1 * $fs->shield_tech));
}


sub hull {
    my ($type, $fs) = @_;

    return int($base_stats{$type}{hull} * (1 + 0.1 * $fs->armor_tech));
}


sub base_speed {
    my ($type, $fs) = @_;

    if ($type eq 'Atl') {
        if ($fs->pulse_tech < 5) {
            return 5000;
        }
        return 10000;
    }
    elsif ($type eq 'Are') {
        if ($fs->warp_tech < 8) {
            return 4000;
        }
        return 5000;
    }

    return $base_stats{$type}{speed};
}


sub engine {
    my ($type, $fs) = @_;

    if ($type eq 'Atl') {
        if ($fs->pulse_tech < 5) {
            return 'jet';
        }
        return 'pulse';
    }
    elsif ($type eq 'Are') {
        if ($fs->warp_tech < 8) {
            return 'pulse';
        }
        return 'warp';
    }

    return $base_stats{$type}{engine};
}
    

sub speed {
    my ($type, $fs) = @_;

    my $engine = engine($type, $fs);
    my $base = base_speed($type, $fs);

    if ($engine eq 'jet') {
        return $base * (1 + 0.1 * $fs->jet_tech);
    }
    elsif ($engine eq 'pulse') {
        return $base * (1 + 0.2 * $fs->pulse_tech);
    }
    elsif ($engine eq 'warp') {
        return $base * (1 + 0.3 * $fs->warp_tech);
    }

}

sub fuel_consumption {
    return $base_stats{$_[0]}{fuel} unless $_[0] eq 'Atl';
    if (defined $_[1] && $_[1]->pulse_tech < 5) {
        return 10;
    }
    return 20;
}

sub full_name        { $base_stats{$_[0]}{fullname} }
sub ore_cost         { $base_stats{$_[0]}{ore} }
sub crystal_cost     { $base_stats{$_[0]}{cry} }
sub hydrogen_cost    { $base_stats{$_[0]}{gen} }
sub is_ship          { $base_stats{$_[0]}{ship} }
sub sort_order       { $base_stats{$_[0]}{sorto} }
sub drop_order       { $base_stats{$_[0]}{dropo} }
sub cargo_space      { $base_stats{$_[0]}{cargo} }
sub all_units        { keys %base_stats }
sub base_weapon      { $base_stats{$_[0]}{weapon} }
sub base_shield      { $base_stats{$_[0]}{shield} }
sub base_hull        { $base_stats{$_[0]}{hull} }


sub stdsort ($$) {
    $base_stats{$_[0]}{sorto} <=> $base_stats{$_[1]}{sorto};
}

sub drop_sort ($$) {
    $base_stats{$_[0]}{dropo} <=> $base_stats{$_[1]}{dropo};
}

# sub speedsort ($$) {
#      $base_stats{$_[0]}{sorto} <=> $base_stats{$_[1]}{sorto};
# }

1;
