
package SFC::Sim;

use strict;
use warnings qw{ all };

# use JSON -convert_blessed_universally;
# use Data::Dumper;
# $Data::Dumper::Terse = 1;
# $Data::Dumper::Indent = 0;

use IPC::Open2;

use SFC::FSArray qw{ serialize deserialize };


sub new {
    my $this = shift;
    my $class = ref($this) || $this;
    my $self = ref($this) ? { %$this } : {
            # defaults
        };
    bless $self, $class;

    $self->{pid} = open2($self->{input}, $self->{output}, @_);

    return $self;
}

sub init {
    my $self = shift;
    my ($attackers, $defenders) = @_;

    print { $self->{output} } 'Attackers: ', serialize($attackers), "\n";
    print { $self->{output} } 'Defenders: ', serialize($defenders), "\n";
}

sub dump {
    my $self = shift;
    my ($attackers, $defenders) = @_;

    print 'Attackers: ', serialize($attackers), "\n";
    print 'Defenders: ', serialize($defenders), "\n";
}

sub start {
    my $self = shift;

    print { $self->{output} } "run\n";
}

sub collect {
    my $self = shift;

    my ($rounds, $att_res, $def_res);

    local $_;
    while (defined(my $line = readline $self->{input})) {
        $_ = $line;

        next if /^$/;
        if (/Success-Index: (.*)$/) {
            $rounds = $1 +0;
            next;
        }
        if (/Attacker-Results: (.*)$/) {
            $att_res = deserialize($1);
            next;
        }
        if (/Defender-Results: (.*)$/) {
            $def_res = deserialize($1);
            next;
        }
        last if /^done$/;
    }

    return $rounds, $att_res, $def_res;
}

sub read_fileno {
    my $self = shift;

    fileno($self->{input});
}


1;
