
package SFC::SimTeam;

use strict;
use warnings qw{ all };

use SFC::Sim;


sub new {
    my $this = shift;
    my $class = ref($this) || $this;

    my ($profiling, $suppress_local) = @_;

    my @sim;
    # my $self = ref($this) ? [ @$this ] : [
    #         # defaults
    #         0, 0, 0, 0, 0, 0, 0, 0
    #     ];
    
    if ($profiling) {
        $sim[0] = SFC::Sim->new('ssh', 'cephas',
                'cd proj/al/sfc; nice perl -d:DProf ./simserv.pl');
    }
    else {
        $sim[0] = SFC::Sim->new('ssh', 'cephas',
                'cd proj/al/sfc; nice ./simserv.pl');
        $sim[1] = SFC::Sim->new('nice', './simserv.pl');
        $sim[2] = SFC::Sim->new('ssh', 'cephas',
                'cd proj/al/sfc; nice ./simserv.pl');
        if (!$suppress_local) {
            $sim[3] = SFC::Sim->new('nice', './simserv.pl');
        }
#         $sim[4] = SFC::Sim->new('ssh', 'duns-scotus',
#                 'cd proj/al/sfc; nice ./simserv.pl');
    }

    my $self = \@sim;
    bless $self, $class;
    return $self;
}


sub dump {
    my $self = shift;
    my ($attackers, $defenders) = @_;

    $self->[0]->dump($attackers, $defenders);
}


sub run {
    my $self = shift;

    my ($attackers, $defenders, $left_to_start, $stats) = @_;

    for my $s (@$self) {
        $s->init($attackers, $defenders);
    }

    my $sel = new IO::Select;
    my %sim;

    for my $s (@$self) {
        my $fd = $s->read_fileno;
        $s->start;
        $sel->add($fd);
        $sim{$fd} = $s;
        --$left_to_start or last;
    }

    while (my @ready = $sel->can_read) {
        for my $fd (@ready) {
            my $s = $sim{$fd};
            $stats->record_data($s->collect);

            if ($left_to_start-- > 0) {
                $s->start;
            }
            else {
                $sel->remove($fd);
            }
        }

        last if ! $sel->count;
    }
}


1;
