
package SFC::Vector;

use strict;
use warnings qw{ all };

use List::Util qw{ reduce sum };
use List::MoreUtils qw{ pairwise each_arrayref };
use POSIX qw{ floor };

use overload
    '+' => \&add,
    '-' => \&diff,
    neg => \&neg,
    '*' => \&scmult,
    '/' => \&scdiv,
    '""' => \&stringify,
    bool => \&bool,
    '==' => \&equal,
    abs => \&norm,
    int => \&grid,
;

sub new {
    my $this = shift;
    my $class = ref($this) || $this;
    my $self = @_ ? [ @_ ] : ref($this) ? [ @$this ] : [ ];
    bless $self, $class;
}

sub add {
    my @sum = pairwise { $a + $b } @{$_[0]}, @{$_[1]};
    bless \@sum, ref($_[0]);
}

sub diff {
    my @diff;
    if ($_[2]) {
        @diff = pairwise { $a - $b } @{$_[1]}, @{$_[0]};
    }
    else {
        @diff = pairwise { $a - $b } @{$_[0]}, @{$_[1]};
    }
    bless \@diff, ref($_[0]);
}

sub zerop {
    ! reduce { $a || $b } @{$_[0]};
}

sub bool {
    ! $_[0]->zerop;
}
    
sub neg {
    my @neg = map { -1 * $_ } @{$_[0]};
    bless \@neg, ref($_[0]);
}

sub grid {
    my @grid = map { int $_ } @{$_[0]};
    bless \@grid, ref($_[0]);
}

sub scmult {
    my @prod = map { $_[1] * $_ } @{$_[0]};
    bless \@prod, ref($_[0]);
}

sub scdiv {
    return undef if $_[2];
    my @quot = map { $_ / $_[1]  } @{$_[0]};
    bless \@quot, ref($_[0]);
}

sub round {
    my @int = map { floor($_ + 0.5) } @{$_[0]};
    bless \@int, ref($_[0]);
}

sub ceil {
    my @ceil = map { POSIX::ceil $_ } @{$_[0]};
    bless \@ceil, ref($_[0]);
}

sub scale_round {
#    my @int = map { my $q = ceil($_ * $_[1]) || 1; floor($_ / $q + 0.5) * $q }
    my @int = map {
            my $q = $_ > 1 ?
                floor((1+$_[1]) ** POSIX::ceil(log($_) / log(1+$_[1]))) : 1;
            floor($_ / $q + 0.5) * $q 
        } @{$_[0]};
    bless \@int, ref($_[0]);
}

sub clamp {
    my @res = pairwise { $a < 0 ? 0 : $a > $b ? $b : $a } @{$_[0]}, @{$_[1]};
    bless \@res, ref($_[0]);
}

sub stringify {
    join ':', @{$_[0]};
};

sub equal {
    my $ea = each_arrayref $_[0], $_[1];
    my $eq = 1;
    while ((my ($a, $b) = $ea->()) && $eq) {
        $eq &&= $a == $b;
    }
    return $eq;
}

sub norm {
    unshift @{$_[0]}, 0;
    my $norm = sqrt(reduce { $a + $b ** 2 } @{$_[0]});
    shift @{$_[0]};
    return $norm;
}

sub dot {
    return sum pairwise { $a * $b } @{$_[0]}, @{$_[1]};
}

1;
