var seconds = 1000;
var minutes = 60*seconds;
var hours = 60*minutes;
var fleet_started = false;
var lucky_draw_rolled = false;
var lucky_draw_selected = {};

var timers = {};

var processed_bookmark_callback = false;

function timeString(h, m, s) {
  return h.toPaddedString(2) + ':' + m.toPaddedString(2) + ':' + s.toPaddedString(2);
}

function makeTimer(id, seconds_from_start, seconds_until_end, callback) {
  if (!timers[id]) {
    timers[id] = { 'retries': 0 }
  }

  var start_time = new Date();
  start_time.setSeconds(start_time.getSeconds() - seconds_from_start);

  var end_time = new Date();
  end_time.setSeconds(end_time.getSeconds() + seconds_until_end);

  updateTimer(start_time, end_time, id, callback);
}

function updateTimer(start_time, end_time, id, callback) {
  var now = new Date();
  var t = end_time - now;

  progress = $(id + '_progress');
  percent = $(id + '_percent');
  countdown = $(id + '_countdown');
  timer_text = $(id + '_timer_text');

  if (!progress || !percent) {
    return;
  }

  if (t < 0) {
    if (timers[id]['retries'] == 0) {

      if(callback) {
        setTimeout(callback, 5000 * Math.pow(2, timers[id]['retries']));
        timers[id]['retries'] += 1
        timer_text.innerHTML = "updating...";
      }
      else {
        timer_text.innerHTML = "-";
      }

    } else {
      timer_text.innerHTML = "try again later";
    }

    progress.setStyle({ width: "100%" });

    return;
  }

  var h = Math.floor(t / hours);
  t -= h*hours;

  var m = Math.floor(t / minutes);
  t -= m*minutes;

  var s = Math.floor(t / seconds);

  pct_completed = ((now - start_time) / (end_time - start_time))
  pct = Math.floor(100 * pct_completed) + "%";
  progress.setStyle({ width: pct });
  percent.innerHTML = pct
  countdown.innerHTML = timeString(h, m, s);
  update_speedup_cost(id, (end_time - now));

  if (timers[id]['handle']) { clearTimeout(timers[id]['handle']); }
  timers[id]['handle'] = setTimeout(function() { updateTimer(start_time, end_time, id, callback) }, 1000);
}

function update_speedup_cost(id, time_left) {
  credits_per_second = $('speedup_' + id + '_credits_per_second');
  if (credits_per_second == null) {
    return;
  }

  cps = parseFloat(credits_per_second.innerHTML);
  new_cost = Math.ceil(cps * time_left / 1000.0);
  $('speedup_' + id + '_cost').innerHTML = format_number(new_cost);
}

function format_number(n) {
  n += '';
  var reg = /(\d+)(\d{3})/;

  while (reg.test(n)) {
    n = n.replace(reg, '$1' + ',' + '$2');
  }

  return n;
}

function selectPriorityTarget(defense_id) {
  var selectedTarget = $('priority_target_' + defense_id);
  if (selectedTarget.checked) {
    selectedTarget.checked = '';
    $('target_' + defense_id).removeClassName('selected');
  } else {
    selectedTarget.checked = 'checked';
    updateSelectedTarget(defense_id);
  }
}

function updateSelectedTarget(defense_id) {
  $$('.defense_image').each(function(image) {
    if (image.id == 'target_' + defense_id)
      image.addClassName('selected');
    else
      image.removeClassName('selected');
    });
}

function updateLaunchButton(max_missiles) {
  var form_disabled = $('missile_strike_popup').hasClassName('disabled');
  if($('missile_quantity').value > 0 && $('missile_quantity').value <= max_missiles && !form_disabled) {
    $('launch_button').enable();
    $('missile_quantity').removeClassName('important');

  } else {
    if (!form_disabled) {
      $('missile_quantity').addClassName('important');
    }
    $('launch_button').disable();
  };
}

function disable_launch_button() {
  $('launch_button').disable();
  $('missile_strike_popup').addClassName('disabled');
}

function otherDir(dir) {
  return dir == 'from' ? 'to' : 'from'
}

function selectWorker(dir, id) {
  var worker = $(dir + '_worker_' + id);
  var selected = worker.hasClassName('selected');

  worker.toggleClassName('selected');
  $(dir + '_worker_checkbox_' + id).checked = worker.hasClassName('selected');

  updateAssignButton();
  updateWorkerSlotsMessages();
}

function updateAssignButton() {
  if (workerSlotsTooFull('from') || workerSlotsTooFull('to') || selectedWorkerCount() <= 0) {
    $('assign_worker_button').disable();
  } else {
    $('assign_worker_button').enable();
  }
}

function selectedWorkerCount() {
  return $$('#from_worker_selector .worker.selected').length;
}

function workerSlotsTooFull(dir) {
  var worker_slots = $(otherDir(dir) + '_worker_slots');
  if (!worker_slots) { return false; }

  var used_slots = $$('#' + dir + '_worker_selector .worker.selected').length -
    $$('#' + otherDir(dir) + '_worker_selector .worker.selected').length;

  return used_slots > $F(worker_slots);
}

function updateWorkerSlotsMessages() {
  updateWorkerSlotsMessage('from');
  updateWorkerSlotsMessage('to');
}

function updateWorkerSlotsMessage(dir) {
  var worker_slots = $(otherDir(dir) + '_worker_slots');
  if (!worker_slots) { return; }

  var worker_slots_span = $(otherDir(dir) + '_worker_slots_span');
  if (!worker_slots_span) { return; }

  var used_slots = $$('#' + dir + '_worker_selector .worker.selected').length -
    $$('#' + otherDir(dir) + '_worker_selector .worker.selected').length;

  var slots = $F(worker_slots) - used_slots;
  worker_slots_span.innerHTML = slots;

  if (slots < 0) { worker_slots_span.addClassName('important'); }
  else { worker_slots_span.removeClassName('important'); }
}

function clearSelectedWorkers() {
  $$('.worker.selected').each(function(worker) {
    worker.removeClassName('selected');
  });
}

function get_absolute_top(elem) {
  var top = 0;
  do { top += elem.offsetTop; } while ((elem = $(elem.offsetParent)) && (elem.getStyle("position") != "relative"));

  return top;
}

function get_above_absolute_top(elem) {
  var top = 0;

  while ((elem = elem.offsetParent) && (elem.getStyle("position") != "relative")) {}
  do { top += elem.offsetTop; } while ((elem = $(elem.offsetParent)));

  return top;
}

function get_absolute_left(elem) {
  var left = 0;

  do { left += elem.offsetLeft; } while ((elem = $(elem.offsetParent)) && (elem.getStyle("position") != "relative"));

  return left;
}

function remove_all(selector) {
  $$(selector).each(function(item){ item.remove() });
}

function hide_all(css_class) {
  $$(css_class).each(function(item){ item.hide() });
}

function move_popup_near(source, popup, align_top, store_popup) {
  if(store_popup) {
    $(popup).setStyle({
      left: ('220px')
    });
  } else {
    $(popup).setStyle({
      left: ('95px')
    });
  }

  if(align_top) {
    var offset = 0;
  } else {
    var offset = -($(popup).clientHeight / 2);
  }

  $(popup).clonePosition(source, { setWidth : false, setHeight : false, setLeft : false, offsetTop : offset })
}

function setup_close_link() {
  var close_button = $('close_link');

  close_button.focus();

  close_button.onkeydown = function(event) {
      var key;
      if (window.event) { key = window.event.keyCode; }
      else { key = event.which; }

      if (key == 27) { // ESC
          this.onclick();
          return false;
      }

      return true;
  }
}

function max_ships(id) {
  var max = $(id + '_max');
  var batch_size = $('batch_size');

  if (max) {
    if (batch_size) {
      batch_size = $F('batch_size');
      return Math.floor(parseInt($(id + '_max').innerHTML) / batch_size);

    } else {
      return parseInt($(id + '_max').innerHTML);
    }

  } else {
    return 0;
  }
}

function check_max_ships(id) {
  check_max(id, max_ships(id));
}

function assign_max_ships(id) {
  $(id).value = max_ships(id);
}

function select_all_ships() {
  $$('.ship_quantity').each(function(ship_quantity) {
    assign_max_ships(ship_quantity.id)
  });
}

function select_no_ships() {
  $$('.ship_quantity').each(function(ship_quantity) {
    ship_quantity.value = 0;
  });

  select_no_cargo();
}

function update_dispatch_button() {
  var disabled = false;
  var total_quantity = 0;

  $$('.ship_quantity').each(function(ship_quantity) {
    var max = max_ships(ship_quantity.id);
    var quantity = parseInt(ship_quantity.value);
    total_quantity += quantity

    if(quantity < 0 || quantity > max) {
      disabled = true;
    }
  });

  $$('.required_ship').each(function(required_ship) {
    var ship_quantity = $('ship_quantity_' + $F(required_ship));
    if (!ship_quantity || parseInt($F(ship_quantity)) == 0) {
      disabled = true;
    }
  });

  if(total_quantity <= 0) {
    disabled = true;
  }

  if ($('send_ore')) { if ($F('send_ore') > max_cargo('ore')) { disabled = true; } }
  if ($('send_crystal')) { if ($F('send_crystal') > max_cargo('crystal')) { disabled = true; } }
  if ($('send_hydrogen')) { if ($F('send_hydrogen') > max_cargo('hydrogen')) { disabled = true; } }

  if ($('trade_amount')) {
    var trade_amount = parseInt($F('trade_amount'));
    var max_amount = parseInt($F('max_' + $('trade_type').value));
    if (trade_amount > total_capacity() || trade_amount <= 0 || trade_amount > max_amount || isNaN(trade_amount)) {
      disabled = true;
      $('trade_amount').addClassName('important');
    } else {
      $('trade_amount').removeClassName('important');
    }

    if (unbalanced_trade()) {
      disabled = true;
      $('receive_amount').addClassName('important');
    } else {
      $('receive_amount').removeClassName('important');
    }
  }

  if ($('cargo_for_trade')) {
    if ($F('cargo_for_trade') > total_capacity()) {
      disabled = true;
      $('to_trade').addClassName('important');
      $('insufficient_cargo').show();
    } else {
      $('to_trade').removeClassName('important');
      $('insufficient_cargo').hide();
    }
  }

  if ($('workers_assigned') != null) {
    if (parseInt($('workers_assigned').innerHTML) > parseInt($('workers_total').innerHTML)) {
      disabled = true;
      $('workers_assigned').addClassName('important');
    } else {
      $('workers_assigned').removeClassName('important');
    }
  }

  if(disabled || $('assign_fleet_form').hasClassName('disabled')) {
    $('assign_button').disable();
  } else {
    $('assign_button').enable();
  }
}

function disable_assign_fleet_form() {
  $('assign_button').disable();
  $('assign_fleet_form').addClassName('disabled');
}

function fuel_consumption() {
  var fuel_consumption = parseInt($('task_consumption').innerHTML);
  if (isNaN(fuel_consumption)) { fuel_consumption = 0; }

  return fuel_consumption;
}

function total_capacity() {
  var total_capacity = 0;

  $$('.ship_quantity').each(function(ship_quantity) {
    var quantity = parseInt(ship_quantity.value);
    var cargo_capacity = parseInt($(ship_quantity.id + '_cargo_capacity').innerHTML);
    var max = max_ships(ship_quantity.id);

    total_capacity += Math.min(quantity, max) * cargo_capacity;
  });

  return total_capacity - fuel_consumption();
}

function total_allocated() {
  var total_allocated = 0;

  $$('.send_resource').each(function(send_resource) {
    var resource = /send_(.*)/.exec(send_resource.id)[1]; // HACK
    var allocated = Math.min(parseInt(send_resource.value), $F('max_' + resource));

    if (!isNaN(allocated)) {
      total_allocated += Math.max(allocated, 0);
    }
  });

  return total_allocated;
}

function max_cargo(resource) {
  var max = $F('max_' + resource);
  if (resource == 'hydrogen') {
    max -= fuel_consumption();
  }

  var allocated = parseInt($('send_' + resource).value);
  if (isNaN(allocated)) { allocated = 0; }

  var space_available = total_capacity() - total_allocated();

  return Math.max(Math.min(max, allocated + space_available), 0);
}

function select_max_cargo(resource) {
  if (!$('send_' + resource)) { return; }

  $('send_' + resource).value = max_cargo(resource);
}

function select_max_batches(batch_id) {
  if(!batch_id) { batch_id = 'batch_size'; }

  var last_child = $(batch_id).select(":last-child")[0];
  last_child.selected = true;
}

function select_no_cargo() {
  if ($('send_ore')) { $('send_ore').value = 0 }
  if ($('send_crystal')) { $('send_crystal').value = 0 }
  if ($('send_hydrogen')) { $('send_hydrogen').value = 0 }

  check_max_cargo('ore');
  check_max_cargo('crystal');
  check_max_cargo('hydrogen');
}

function check_max_cargo(resource) {
  if (!$('send_' + resource)) { return; }

  check_max('send_' + resource, max_cargo(resource));
}

function check_max(id, max) {
  if (!$(id)) { return; }

  if ($(id).value > max || $(id).value < 0) {
    $(id).addClassName('important');

  } else {
    $(id).removeClassName('important');
  }
}

function unbalanced_trade() {
  ratios = get_trade_ratio_range($('trade_type').value, $('receive_type').value);

  min_trade = $F('trade_amount') * ratios['min'];
  max_trade = $F('trade_amount') * ratios['max'];
  receive = $F('receive_amount');

  return ( receive > Math.max(min_trade, max_trade) || receive < Math.min(min_trade, max_trade) );
}

function get_trade_ratio_range(send, receive) {
  var ratios = new Array();

  switch(send) {
    case 'ore':
      switch(receive) {
        case 'crystal':
          ratios['min'] = 1/2;
          ratios['max'] = 1/1.5;
        break;

        case 'hydrogen':
          ratios['min'] = 1/3;
          ratios['max'] = 1/2;
        break;
      }
    break;

    case 'crystal':
      switch(receive) {
        case 'ore':
          ratios['min'] = 1.5;
          ratios['max'] = 2;
        break;

        case 'hydrogen':
          ratios['min'] = 1/2;
          ratios['max'] = 1;
        break;
      }
    break;

    case 'hydrogen':
      switch(receive) {
        case 'ore':
          ratios['min'] = 2;
          ratios['max'] = 3;
        break;

        case 'crystal':
          ratios['min'] = 1;
          ratios['max'] = 2;
        break;
      }
    break;
  }

  return ratios;
}

function set_all_alliance_members_checks(check_value) {
  $$('.alliance_member_check_box').each(function(alliance_member) {
    alliance_member.checked = check_value;
  });
}

function disable_ajax_links() {
  $$('.ajax_link .active').each(function(elem) { elem.hide(); });
  $$('.ajax_link .waiting').each(function(elem) { elem.show(); });
}

function enable_ajax_links() {
  $$('.ajax_link .waiting').each(function(elem) { elem.hide(); });
  $$('.ajax_link .active').each(function(elem) { elem.show(); });
}

function disable_buy(level, disabled) {
  if(disabled) {
    $$('.remote_buy_link.buy_link.level_' + level).each(function(e) { e.hide(); });
    $$('.redirect_to.buy_link.level_' + level).each(function(e) { e.show(); });

  } else {
    $$('.redirect_to.buy_link.level_' + level).each(function(e) { e.hide(); });
    $$('.remote_buy_link.buy_link.level_' + level).each(function(e) { e.show(); });
  }
}

function add_user_to_message_recipients(buddy_id) {
  $('checkbox_' + buddy_id).click();
  message_users_on_check(buddy_id);
}

function message_users_on_check(buddy_id) {
  var buddyCheckbox = $('checkbox_' + buddy_id);
  var buddyRow = $('buddy_' + buddy_id);
  if (buddyCheckbox.checked) {
    buddyRow.addClassName('selected');
  } else {
    buddyRow.removeClassName('selected');
  }
}

function check_all(className, checked) {
  $$('.' + className).each(function(cb) {
    cb.checked = checked;
    })
}

function highlight_check_all_option(name) {
  ['check_all', 'check_none'].each(function(id) {
      $(id).removeClassName('selected');
  });
  if (name) {
    $(name).addClassName('selected');
  }
}

function check_all_option_clicked(id, total_message_count) {
  highlight_check_all_option(id);
  check_all('message_checkbox', (id == 'check_none' ? '' : 'checked'));

  message_checkbox_change();
  $('check_all_pages').value = 'false';
  if (id == 'check_all' && total_message_count > 20){
    $('check_all_pages_message').show();
    $('check_all_pages_link').show();
    $('clear_selection_link').hide();
  } else {
    $('check_all_pages_message').hide();
  }
}

function check_all_pages(total_message_count) {
  highlight_check_all_option(null);
  check_all('message_checkbox', 'checked');
  $('check_all_pages').value = 'true';
  $('selected_message_count').innerHTML = total_message_count.toString();
  $('check_all_pages_link').hide();
  $('clear_selection_link').show();
}

function message_checkbox_change() {
  var count = 0;
  var checkedCount = 0;
  $$('.message_checkbox').each(function(cb) {
      count += 1;
      if (cb.checked) {
        checkedCount += 1;
      }
  });
  if (checkedCount == 0) {
    highlight_check_all_option('check_none');
  } else if (count == checkedCount) {
    highlight_check_all_option('check_all');
  } else {
    highlight_check_all_option();
  }

  if ($('check_all_pages').value == 'true') {
    checkedCount = $('total_message_count').innerHTML - (count - checkedCount);
  }

  $('selected_message_count').innerHTML = checkedCount.toString();
}

function update_message_buddylist() {
  $$('.buddy_checkbox').each(function(cb) {
    message_users_on_check(cb.value)
  })
}

function updateSendButton() {
  var form = $('new_user_message');
  var inputs = form.getInputs('', 'recipients[]');
  var okayToSend = false;

  inputs.each(function(input) {
    if (input.checked || input.type == "hidden") {
      okayToSend = true;
    }
  });

  if (okayToSend) {
    $('user_message_submit').enable();
  } else {
    $('user_message_submit').disable();
  }
}

function selectTab(tab) {
  $$('.tab.selected').each(function (elem) { elem.removeClassName('selected') });
  $$('.tab-content.selected').each(function (elem) { elem.removeClassName('selected') });

  $('tab-' + tab).addClassName('selected');
  $('tab-content-' + tab).addClassName('selected');
}

function showAllPosts() {
  $$('.post').each(function (post) { post.show(); });
  $$('.space').each(function (post) { post.show(); });

  $('see_all').hide();
  $('see_all_space').hide();
}

function incrementWidget(id, to_add, min, max) {
  if (!$(id)) { return; }

  var value = parseInt($F(id));
  if (isNaN(value)) { value = 0; }

  value += to_add;
  if ((min != undefined) && (value < min)) { value = min; }
  if ((max != undefined) && (value > max)) { value = max; }

  $(id).value = value;
}

function updateTrade() {
  var trade_res = getSelectedRadioButton('resource_to_trade');
  var receive_res = getSelectedRadioButton('resource_to_receive');

  $$('input[name=resource_to_receive]').each(function(button){button.enable();})
  if ($('resource_to_receive_' + trade_res)) {
    $('resource_to_receive_' + trade_res).disable();
    $('resource_to_receive_' + trade_res).checked = false;
  }

  var to_trade = $F('trade')

  if (receive_res) {
    $('receive').value = calculateTrade(to_trade, trade_res, receive_res)
  } else {
    $('receive').value = 'none';
  }

  checkTradeInputs();
}

function calculateTrade(to_trade, trade_res, receive_res) {
  var trade_ratio, receive_ratio;

  switch(trade_res) {
    case 'ore':
      trade_ratio = 2.5;
      break;
    case 'crystal':
      trade_ratio = 1.5;
      break;
    case 'hydrogen':
      trade_ratio = 1;
      break;
  }
  switch(receive_res) {
    case 'ore':
      receive_ratio = 2.5;
      break;
    case 'crystal':
      receive_ratio = 1.5;
      break;
    case 'hydrogen':
      receive_ratio = 1;
      break;
  }

  return Math.floor(to_trade * receive_ratio / trade_ratio);
}

function getSelectedRadioButton(name) {
  var selected;
  $$('input[name=' + name + ']').each(function (button) {
    if (button.checked) {
      selected = button.value;
    }
  })
  return selected;
}

function checkTradeInputs() {
  var trade_res = getSelectedRadioButton('resource_to_trade');
  var receive_res = getSelectedRadioButton('resource_to_receive');
  var button_state = true;

  if ($('trade').value) {
    if (parseInt($('trade').value) > parseInt($('max_' + trade_res + '_to_trade').value)) {
      $('trade').addClassName('important');
      button_state = false;
    } else {
      $('trade').removeClassName('important');
    }

  } else {
    button_state = false;
  }

  if (receive_res && $('receive').value) {
    if (parseInt($('receive').value) > parseInt($('max_' + receive_res + '_to_receive').value)) {
      $('receive').addClassName('important');
      button_state = false;
    } else {
      $('receive').removeClassName('important');
    }

  } else {
    button_state = false;
  }

  if (trade_res == receive_res) {
    button_state = false;
  }

  setTradeButton(button_state);
}

function setTradeButton(button_state) {
  if (button_state) {
    $('initiate_trade').enable();
  } else {
    $('initiate_trade').disable();
  }
}

function maxTrade() {
  var trade_res = getSelectedRadioButton('resource_to_trade');
  var receive_res = getSelectedRadioButton('resource_to_receive');

  var max_selected_resource = parseInt($('max_' + trade_res + '_to_trade').value);
  var max_receive = parseInt($('max_' + receive_res + '_to_receive').value);

  if (max_receive < 0) {
    max_receive = 0;
  }

  var receive = calculateTrade(max_selected_resource, trade_res, receive_res);
  if (receive > max_receive) {
    max_selected_resource = calculateTrade(max_receive, receive_res, trade_res);
    receive = calculateTrade(max_selected_resource, trade_res, receive_res);
  }

  $('trade').value = max_selected_resource;
  $('receive').value = receive;
}

function assign_workers_to_fleet() {
  var selected_workers = $$(".worker.selected input[name='from_workers[]']");
  $('workers_assigned').update(selected_workers.size());
  $('worker_ids').value = selected_workers.map(function(e){return e.value}).join(' ');
}

function select_workers() {
  if ($('worker_ids').value == "") {
    return;
  }

  var workers = $('worker_ids').value.split(' ');
  workers.each(function(worker){
    $('from_worker_checkbox_' + worker).click();
  })
}

function hide_fleet_popup() {
  $$('#assign_fleet.popup').each(function(link) {
    link.hide();
  })
}

function show_fleet_popup() {
  $$('#assign_fleet.popup').each(function(link) {
    link.show();
  })
}

function get_inner_html(id) {
  if($(id)) {
    return parseInt($(id).innerHTML);
  }
}

function get_value(id) {
  if($(id)) {
    return parseInt($(id).value)
  }
}

function update_fleet_info() {
  if (!$('assign_fleet')) { return false; }

  if($('distance')) {
    var distance = parseInt($F('distance'))
  }
  else {
    var distance = update_distance();
    update_selected_planet();
  }

  update_duration(distance);

  update_group_defend_cost();

  show_suitable_missions();

  check_all_max_ships();
  check_all_max_cargo();
  update_dispatch_button();
}

function update_distance() {
  var distance = 0

  var p = get_inner_html('current_planet_position');
  var ss = get_inner_html('current_planet_solar_system');
  var g = get_inner_html('current_planet_galaxy');

  var tar_p = get_value('planet');
  var tar_ss = get_value('solar_system');
  var tar_g = get_value('galaxy');

  if(p == tar_p && ss == tar_ss && g == tar_g) {
    distance = 5
  }
  else if(ss == tar_ss && g == tar_g) {
    distance = 1000 + Math.abs(p - tar_p) * 5
  }
  else if(g == tar_g) {
    distance = 2700 + Math.abs(ss - tar_ss) * 95
  }
  else {
    distance = Math.abs(g - tar_g) * 20000
  }

  $('task_distance').innerHTML = distance;
  return distance;
}

function get_min_speed() {
  var speed = 0
  $$('.ship_quantity').each(function(ship_quantity) {
    var quantity = parseInt(ship_quantity.value);
    if(quantity > 0) {
      if(speed == 0) {
        speed = get_inner_html(ship_quantity.id + '_speed');
      }
      else {
        speed = Math.min(speed, get_inner_html(ship_quantity.id + '_speed'));
      }
    }
  });

  return speed;
}

function set_task_duration(duration) {
  var h = Math.floor(duration / 3600);
  var minute_duration = duration - h*3600;

  var m = Math.floor(minute_duration / 60);
  var second_duration = Math.ceil(minute_duration - m*60);

  $('task_duration').innerHTML = timeString(h, m, second_duration);
}

function update_duration(distance) {
  var speed = get_min_speed();

  if(speed == 0) {
    $('task_duration').innerHTML = '-';
    $('task_consumption').innerHTML = '-';
  }
  else {
    if($('speed')) {
      var percentage = parseInt($F('speed'));
    }
    else {
      var percentage = 10;
    }

    var base_duration = Math.ceil((35000.0/percentage) * Math.sqrt((distance * 10) / speed) + 10);
    var duration = base_duration;
    if ($('universe_speed')) {
      duration *= parseInt($F('universe_speed')) / 3600.0;
    }
    set_task_duration(duration);

    var total_consumption = 0;
    var factor = $('hydro_commander_present') ? 0.90 : 1.0
    $$('.ship_quantity').each(function(ship_quantity) {
      var quantity = parseFloat(ship_quantity.value);

      if(quantity > 0) {
        var consumption = get_inner_html(ship_quantity.id + '_fuel_consumption');
        var base_speed = get_inner_html(ship_quantity.id + '_speed');

        var ship_speed = (35000.0/(base_duration-10)) * Math.sqrt((distance * 10) / base_speed);

        consumption = consumption * quantity * distance / 35000.0 * Math.pow((ship_speed / 10.0 + 1), 2)
        total_consumption += consumption;
      }
    });

    var adjusted_consumption = total_consumption * factor
    if ($('hydro_commander_present')) {
      $('hydro_commander_notice').innerHTML = Math.ceil(total_consumption - adjusted_consumption) + " Hydrogen Saved By Propulsion Engineer"
    }

    total_consumption *= factor

    $('task_consumption').innerHTML = Math.ceil(adjusted_consumption);
  }
}

function on_the_moon() {
  return $('current_planet_type').innerHTML.strip() == 'moon';
}

function moon_has_warp_gate() {
  return $('current_moon_has_no_warp_gate').innerHTML.strip() != 'true';
}

function target_is_my_moon() {
  return target_is_my_planet() && $F('planet_type') == 'moon';
}

function target_is_my_planet() {
  var is_my_planet = false;

  var p = get_value('planet');
  var ss = get_value('solar_system');
  var g = get_value('galaxy');
  var m = $F('planet_type');

  $$('.planet_drop_down_option').each(function(option) {
    var atts = option.attributes;
    var planet_true = atts['planet'].value == p;
    var solar_true = atts['solar_system'].value == ss;
    var galaxy_true = atts['galaxy'].value == g;
    var type_true = atts['type'].value == m;

    if(planet_true && solar_true && galaxy_true && type_true)
      is_my_planet = true
  });

  $$('.planet_choice .coordinates').each(function(coords){
    var this_g = parseInt(coords.childElements()[0].innerHTML);
    var this_ss = parseInt(coords.childElements()[1].innerHTML);
    var this_p = parseInt(coords.childElements()[2].innerHTML);
    var this_m = coords.childElements()[3].innerHTML;

    if (g == this_g && ss == this_ss && p == this_p && m == this_m) {
      is_my_planet = true;
    }
  });

  return is_my_planet;
}

function update_group_defend_cost() {
  if($('wait_duration')) {
    var wait_duration = parseInt($F('wait_duration'));
    var consumption = get_inner_html('task_consumption');

    if(!consumption) { return; }

    var total_cost = 0

    $$('.ship_quantity').each(function(ship_quantity) {
      var quantity = parseInt(ship_quantity.value);
      total_cost += (get_inner_html(ship_quantity.id + '_group_defend_cost') * quantity);
    });

    var hydrogen_addition = Math.round(total_cost * wait_duration/1.0);

    $('task_consumption').innerHTML = consumption + hydrogen_addition
  }
}

function hide_and_uncheck(id) {
  e = $(id);
  e.down().checked = false;
  e.hide();
}

function check_radio (id) {
  e = $(id);
  e.down().checked = true;
}

function set_default_radio_button(found, radio_id, multiple) {
  if(found) {
    if(!fleet_started) {
      multiple ? $(radio_id).show() : check_radio(radio_id);
      fleet_started = true;
    }
  }
  else {
    hide_and_uncheck(radio_id)
  }
}

function show_suitable_missions() {
  mission_options = $$('.mission_option');
  if(mission_options.length == 0){ return; }

  mission_options.each(function(option){ option.show(); });

  var my_planet = target_is_my_planet();

  if (my_planet) {
    hide_and_uncheck('attack_selector');
  } else {
    hide_and_uncheck('deploy_selector');
  }

  if (!(on_the_moon() && target_is_my_moon() && moon_has_warp_gate())) {
    hide_and_uncheck('warp_selector');
  }

  var dionysus_class_found = false;
  var gaia_class_found = false;
  var probe_class_found = false;

  $$('.ship_quantity').each(function(ship_quantity) {
    var quantity = parseInt(ship_quantity.value);
    ship_key = $(ship_quantity.id + '_key').innerHTML;
    if(quantity > 0) {
      if(ship_key == 'dionysus_class') { dionysus_class_found = true; }
      if(ship_key == 'gaia_class' && my_planet == false) { gaia_class_found = true; }
      if((ship_key == 'hermes_class' || ship_key == 'stealth_probe_class') && my_planet == false) { probe_class_found = true; }
    }
    else {
      if(ship_key == 'dionysus_class') { fleet_started = false; }
      if(ship_key == 'gaia_class' && my_planet == false) { fleet_started = false; }
      if((ship_key == 'hermes_class' || ship_key == 'stealth_probe_class') && my_planet == false) { fleet_started = false; }
    }
  });

  two = (dionysus_class_found ? (gaia_class_found || probe_class_found) : (dionysus_class_found && probe_class_found));
  three = dionysus_class_found && gaia_class_found && probe_class_found;

  multiple = two;
  if (three == true) { multiple = false };
  set_default_radio_button(dionysus_class_found, 'harvest_selector', multiple);
  set_default_radio_button(gaia_class_found, 'colonize_selector', multiple);
  set_default_radio_button(probe_class_found, 'espionage_selector', multiple);

  select_mission();
}

function check_all_max_ships () {
  $$('.ship_quantity').each(function(ship_quantity) {
    check_max_ships(ship_quantity.id);
  });
}

function check_all_max_cargo () {
  check_max_cargo('ore');
  check_max_cargo('crystal');
  check_max_cargo('hydrogen');
}

function clear_selected_planet() {
  $$('.planet_choice').each(function(choice){ choice.removeClassName('selected'); });
  $$('input[name="receive_planet"]').each(function(button){ button.checked = ''; });
}

function update_selected_planet() {
  clear_selected_planet();

  var p = get_value('planet');
  var ss = get_value('solar_system');
  var g = get_value('galaxy');
  var m = $F('planet_type');

  $$('.planet_choice .coordinates').each(function(coords){
    var this_g = parseInt(coords.childElements()[0].innerHTML);
    var this_ss = parseInt(coords.childElements()[1].innerHTML);
    var this_p = parseInt(coords.childElements()[2].innerHTML);
    var this_m = coords.childElements()[3].innerHTML;

    if( g == this_g && ss == this_ss && p == this_p && m == this_m) {
      coords.parentNode.addClassName('selected');
    }
  });
}

function buddy_request_to_pending(id) {
  $$('#friend_entry_' + id + ' .enabled').each(function(e) { e.hide(); })
  $$('#friend_entry_' + id + ' .disabled').each(function(e) { e.show(); })
  $$('#friend_entry_' + id + ' .button').each(function(e) { e.update('Pending'); })
}

function fade_reply_form(message_id) {
  reply_form = $('message_' + message_id + '_body').lastChild.previousSibling;
  reply_form.fade({ duration: 0.4 });
}

function update_receive_type() {
  var receive_options = $('trade_type').options;

  $('receive_type').options.length=0;
  for(i = 0; i < receive_options.length; i++) {
    if ($('trade_type').selectedIndex == i) { continue; }
    try {
      $('receive_type').add(new Option(receive_options[i].text, receive_options[i].value), null);
    } catch(ex) {
      $('receive_type').add(new Option(receive_options[i].text, receive_options[i].value)); //IE ONLY
    }
  }
}

function selectPlanet(planet_id) {
  var selectedPlanet = $('receive_planet_' + planet_id);

  if (selectedPlanet.checked) {
    return;

  } else {
    selectedPlanet.checked = 'checked';
    updateSelectedPlanet(planet_id);
  }
}

function updateSelectedPlanet(planet_id) {
  $$('.planet_choice').each(function(planet) {
    if (planet.id == 'planet_' + planet_id) {
      planet.addClassName('selected');

    } else {
      planet.removeClassName('selected');
    }
  });
}

function updateMessageOnclick(message_id) {
  $(message_id).childElements().each(function(child) {
    if (child.readAttribute('onclick') != null) {
      child.writeAttribute('onclick', '$(\'' + message_id + '_content\').toggle();');
    }
  });
}

function clean_up_table(table_id) {
  $(table_id).childElements().each(function(tbody) {
    if (tbody.childElements().length < 1) {
      tbody.remove();
    }
  });
}

function select_field(input_id) {
  var element = $(input_id);

  element.focus();
  element.select();
}

function load_profile(planet_id) {
  window.location.href="/profile?current_planet=" + planet_id
}

function contact_click(self) {
  self.children[0].click();

  if (self.children[0].checked) {
    self.addClassName('selected');
  } else {
    self.removeClassName('selected')
  }
}

function select_all_contacts() {
  $$('.contact').each(function(c) {
    if (c.children[0].checked) {
    } else {
      contact_click(c);
    }
  });
}

function deselect_all_contacts() {
  $$('.contact').each(function(c) {
    if (c.children[0].checked) {
      contact_click(c);
    } else {
    }
  });
}

function update_prizes(prize) {
  hide_all('.hidden_select');

  if(prize != 'credit') { $(prize + '_select').show(); }
}

function startdrag(t, e) {
    if (e.preventDefault) e.preventDefault(); //line for IE compatibility
    e.cancelBubble = true;
    window.document.onmousemoveOld = window.document.onmousemove;
    window.document.onmouseupOld = window.document.onmouseup;
    window.document.onmousemove=dodrag;
    window.document.onmouseup=stopdrag;
    window.document.draged = t;
    t.dragX = e.clientX;
    t.dragY = e.clientY;
    return false;
}
//move the DIV
function dodrag(e) {

    if (!e) e = event; //line for IE compatibility
    t = window.document.draged;
    t.style.left = (t.offsetLeft + e.clientX - t.dragX)+"px";
    t.style.top = (t.offsetTop + e.clientY - t.dragY)+"px";
    t.dragX = e.clientX;
    t.dragY = e.clientY;
    return false;
}
//restore event-handlers
function stopdrag() {
   window.document.onmousemove=window.document.onmousemoveOld;
   window.document.onmouseup=window.document.onmouseupOld;
}

function toggle_visibility(element_id) {
  var element = $(element_id);
  if (element.getStyle('visibility') == 'visible') {
    element.setStyle({visibility:'hidden'});
  } else {
    element.setStyle({visibility:'visible'});
  }
}

function update_progress() {
  var current_progress = parseInt($('social_progress_bar').getStyle('width'));
  current_progress += 20;
  current_progress += '%';
  $('social_progress_bar').innerHTML = current_progress + " completed";
  $('social_progress_bar').setStyle({width:current_progress});

  if (current_progress == '100%') {
    if ($F('received_social_bonus') == 'false') {
      new Ajax.Request('/social_bar/award_bonus');
    } else {
      $('social_setup_bar').fade({ duration: 0.5 });
    }
  }
}

function rgb_to_hex(r, g, b) {
    var rgb = b | (g << 8) | (r << 16);
    return "#" + pad(rgb.toString(16), 6);
}

function pad(s, len) {
    var to_pad = len - s.length;

    var zeroes = [];
    for(var i = 0; i < to_pad; i++) {
        zeroes[i] = '0'
    }

    return zeroes.join("") + s;
}

function update_rap_orbit(speed_percentage) {
  $('rap_fuel').innerHTML = $F('fuel_' + speed_percentage);
  $('rap_time').innerHTML = $F('time_' + speed_percentage);
}

function update_declare_war(alliance_id) {
  var min_points = parseInt($F('war_min_goal_points'));
  var max_points = parseInt($F('war_max_goal_points'));
  var button = $('declare_war_button_' + alliance_id);
  var current_points = parseInt($F('points_goal_' + alliance_id));

  if (min_points <= current_points && current_points <= max_points) {
    button.enable();
  } else {
    button.disable();
  }
}

function change_badge_category(category) {
  var categories = $$(".badge_category")

  for (var i = 0 ; i < categories.length ; i++)
  {
    categories[i].hide();
  }

  $(category).show();
}

function toggle_fleet_selector(mission_id) {
  $$('#fleet_selector_'+mission_id)[0].toggle();
}

function create_fleet_draggable() {
  new Draggable('assign_fleet', {handle: 'handle'});
}

function create_missile_draggable() {
  new Draggable('missile_strike_popup', {handle: 'handle'});
}

function update_show_password() {
    if($F('show_password') != null) {
        $('password').type = 'text';
    } else {
        $('password').type = 'password';
    }
}

function toggle_existing_players_login() {
    if($('existing_players_tab_content').getStyle('display') == 'none') {
        Effect.SlideDown('existing_players_tab_content', {
            duration: 0.2,
            queue: { position: 'end', scope: 'menuxscope', limit: 1 },
            afterFinish: function() { $('login').focus(); }
        });

    } else {
        Effect.SlideUp('existing_players_tab_content', { duration: 0.2, queue: { position: 'end', scope: 'menuxscope', limit: 1 } });
    }

    return false;
}

function pretty_number(n) {
    var s = n.toString();
    var result = "";

    while(s.length > 3) {
        result = "," + s.slice(-3) + result;
        s = s.slice(0, -3);
    }

    return s + result;
}

function update_system_time() {
    var now = new Date();
    var delta = now - page_load_time;

    var new_date = new Date(system_time_start + delta);

    $('system_time_value').innerHTML = timeString(new_date.getUTCHours(), new_date.getUTCMinutes(), new_date.getUTCSeconds());
}

function update_resources() {
    var now = new Date();
    var delta = (now - resources_load_time) / 1000.0;

    var o = Math.floor(starting_ore + delta * ore_rate);
    var c = Math.floor(starting_crystal + delta * crystal_rate);
    var h = Math.floor(starting_hydrogen + delta * hydrogen_rate);

    $('resource_ore').innerHTML = pretty_number(Math.min(o, ore_max));
    $('resource_crystal').innerHTML = pretty_number(Math.min(c, crystal_max));
    $('resource_hydrogen').innerHTML = pretty_number(Math.min(h, hydrogen_max));
}

function hide_planet_summaries() {
    $$('.summary_tab').each(function(e) { e.removeClassName('selected'); });
    $$('.planet_summary').each(function(elem) { elem.hide(); });
}

function show_planet_summary(id) {
    hide_planet_summaries();

    $('summary_tab_' + id).addClassName('selected');
    $('planet_summary_' + id).show();
}

function hide_planet_actions() {
    $$(".new_galaxy .actions").each(function(e) { e.hide(); });
}

function show_planet_actions(id) {
    hide_planet_actions();

    $(id + "_actions").show();
}

function hide_avatars() {
  $$('.profile_image').each(function(e) {e.hide();});
}

function show_avatar(key) {
  hide_avatars();
  $(key).show();
}

function reset_avatar() {
  hide_avatars();
  $($F('set_avatar')).show();
}

function choose_avatar_no_show(id) {
  $('choice_' + id).checked = true;
  imgs = document.getElementsByClassName('avatar_highlighted');
  for(var x in imgs) {
    imgs[x].className = '';
  }
}

function choose_avatar(id) {
  show_avatar(id);
  choose_avatar_no_show(id);
  $('set_avatar').value = id;
}

function disable_avatar_choice () {
  if (document.getElementById('submit_avatar_choice').parentNode.className.indexOf('enabled') == -1)
    return false;
  document.getElementById('submit_avatar_choice').parentNode.className = document.getElementById('submit_avatar_choice').parentNode.className.replace('enabled', 'disabled');
  return true;
}

function lucky_draw_roll(item1, item2) {
    skip_animation_flag = false;
    set_lucky_draw_rolled(true);
    lucky_draw_clear_selected();

    $('rewards').appear({ duration: 0.5 });
    $('item_won').fade({ duration: 0.5 });
    $$('.promoted_reward').each(function(item) { item.fade({ duration: 0.5 }); });
    $$('.reward_choice_slot').each(function(item) { item.fade({ duration: 0.5 }); });
    $$('.choice_button').each(function(item) { item.disabled = false; });
    disable_element('roll_button');

    var item_jitter = (Math.random() < 0.5);
    var time_jitter = (Math.random() < 0.5);

    var on_complete = n_callback(2, function() {
        lucky_draw_promote_items(item1, item2);
    });

    lucky_draw_roll_item(0, (item_jitter ? item1 : item2), (time_jitter ? 1 : 1),
      [0, 1, 2, 3, 7, 6, 5, 4, 8, 9, 10, 11, 15, 14, 13, 12, 8, 9, 10, 11, 7, 6, 5, 4], on_complete);

    lucky_draw_roll_item(1, (item_jitter ? item2 : item1), (time_jitter ? 1 : 1),
      [15, 11, 7, 3, 2, 6, 10, 14, 13, 9, 5, 1, 0, 4, 8, 12, 13, 9, 5, 1, 2, 6, 10, 14], on_complete);
}

function disable_element(elem) {
    $(elem).disable();
    $(elem).addClassName('disabled');
}

function enable_element(elem) {
    $(elem).enable();
    $(elem).removeClassName('disabled');
}

function lucky_draw_roll_item(id, slot, min_rotations, pattern, callback) {
    var total_steps = pattern.length * min_rotations + $A(pattern).indexOf(slot);

    lucky_draw_select_item(id, pattern[0]);
    lucky_draw_spin(id, slot, pattern, 1, total_steps, callback);
}

function lucky_draw_spin(id, slot, pattern, step, total_steps, callback) {
    var sel = $('selection_' + id);
    var step = (step % pattern.length);

    var last_pos = pattern[(step-1 < 0) ? (pattern.length-1) : (step-1)];
    var last_slot = $('reward_' + last_pos);
    var last_offset = last_slot.positionedOffset();

    var next_pos = pattern[step];
    var next_slot = $('reward_' + next_pos);
    var next_offset = next_slot.positionedOffset()

    var start_point = { top: (last_offset.top - 9), left: (last_offset.left - 9) };
    var end_point = { top: (next_offset.top - 9), left: (next_offset.left - 9) };
    var mid_point = { top: (start_point.top + (end_point.top - start_point.top)/2),
                      left: (start_point.left + (end_point.left - start_point.left)/2) }

    if(total_steps == 0) {
        callback();

    } else {
        var duration = (total_steps > 15 ? 0.05 : 0.05 + ((16 - total_steps) * 0.03));
        if(total_steps == 1) duration = 1.5;

        new Effect.Move(sel, { x: mid_point.left, y: mid_point.top, duration: duration/2, mode: 'absolute',
            transition: Effect.Transitions.linear,
            afterFinish: function() {
                lucky_draw_move_selection(id, next_pos);

                new Effect.Move(sel, { x: end_point.left, y: end_point.top, duration: duration/2, mode: 'absolute',
                    transition: Effect.Transitions.linear,
                    afterFinish: function () { lucky_draw_spin(id, slot, pattern, step + 1, total_steps - 1, callback); } });
            } });
    }
}

function lucky_draw_promote_items(id1, id2) {
    var pos0 = $('rewards_container').positionedOffset();
    var pos1 = $('reward_' + id1).positionedOffset();
    var pos2 = $('reward_' + id2).positionedOffset();

    $('promoted_reward_1').setStyle({ top: (pos1.top + pos0.top + 2) + 'px', left: (pos1.left + pos0.left + 2) + 'px',
        width: '50px', height: '50px', paddingTop: '0px', paddingBottom: '0px', paddingLeft: '0px', paddingRight: '0px',
        borderRadius: '6px' });
    $('promoted_image_1').src = $('reward_image_' + id1).src;
    $('promoted_image_1').title = $('reward_image_' + id1).title;
    $('promoted_buffer_2').src = $('reward_image_' + id1).src;
    $('promoted_buffer_2').title = $('reward_image_' + id1).title;
    $('promoted_reward_name_1').innerHTML = $('reward_name_' + id1).innerHTML;
    $('promoted_reward_name_buffer_2').innerHTML = $('reward_name_' + id1).innerHTML;
    $('promoted_reward_name_1').hide();
    $('promoted_reward_name_buffer_2').hide();
    $('promoted_reward_1').show();
    $('promoted_reward_1').style['opacity'] = 1.0;
    new Effect.Opacity('promoted_reward_1', {duration: 0.0, to: 1.0});

    $('promoted_reward_2').setStyle({ top: (pos2.top + pos0.top + 2) + 'px', left: (pos2.left + pos0.left + 2) + 'px',
        width: '50px', height: '50px', paddingTop: '0px', paddingBottom: '0px', paddingLeft: '0px', paddingRight: '0px',
        borderRadius: '6px' });
    $('promoted_image_2').src = $('reward_image_' + id2).src;
    $('promoted_image_2').title = $('reward_image_' + id2).title;
    $('promoted_buffer_1').src = $('reward_image_' + id2).src;
    $('promoted_buffer_1').title = $('reward_image_' + id2).title;
    $('promoted_reward_name_2').innerHTML = $('reward_name_' + id2).innerHTML;
    $('promoted_reward_name_buffer_1').innerHTML = $('reward_name_' + id2).innerHTML;
    $('promoted_reward_name_2').hide();
    $('promoted_reward_name_buffer_1').hide();
    $('promoted_reward_2').show();
    $('promoted_reward_2').style['opacity'] = 1.0;
    new Effect.Opacity('promoted_reward_2', {duration: 0.0, to: 1.0});

    fade_rewards(0.3, function() {
        $('rewards').hide();
        new Effect.Opacity('rewards', { from: 0.0, to: 1.0, duration: 0.0 });

        if(skip_animation_flag) {
            var callback = n_callback(3, function() {
              lucky_draw_flip_cards(function() {});
            });

        } else {
            var callback = n_callback(2, function() {
                $('promoted_reward_name_1').appear({ duration: 0.3 });
                $('promoted_reward_name_2').appear({ duration: 0.3 });

                var shuffle_callback = n_callback(2, function() {
                  lucky_draw_flip_cards(function() {});
                });

                setTimeout(function() {

                    lucky_draw_shuffle(1, 5, 13, 15, 0.5, shuffle_callback);
                    lucky_draw_shuffle(2, 5, 13, 15, 0.5, shuffle_callback);
                }, 1000);
            });
        }

        new Effect.Morph('promoted_reward_1', { style: { width: 50 + 'px', height: 50 + 'px', left: '91px', top: '110px',
            paddingTop: '82.5px', paddingBottom: '82.5px', paddingLeft: '50px', paddingRight: '50px', borderRadius: '20px'
          }, duration: 0.5, afterFinish: callback });
        new Effect.Morph('promoted_reward_2', { style: { width: 50 + 'px', height: 50 + 'px', left: '267px', top: '110px',
            paddingTop: '82.5px', paddingBottom: '82.5px', paddingLeft: '50px', paddingRight: '50px', borderRadius: '20px'
          }, duration: 0.5, afterFinish: callback });

        if(skip_animation_flag) {
            setTimeout(callback, 2000);
        }
    });
}

function slide_names(opts) {
    if(!opts) opts = {};
    if(!opts['id']) opts['id'] = 1;
    if(!opts['x']) opts['x'] = 0;
    if(!opts['y']) opts['y'] = 0;
    if(!opts['duration']) opts['duration'] = 0.3;
    if(!opts['callback']) opts['callback'] = function() {};

    var slide_callback = n_callback(2, opts['callback']);

    var params = { mode: 'absolute', x: opts['x'], y: opts['y'], transition: Effect.Transitions.linear, duration: opts['duration'],
      afterFinish: opts['callback'] }

    new Effect.Move('promoted_reward_name_' + opts['id'], params);
    new Effect.Move('promoted_reward_name_buffer_' + (opts['id'] == 1 ? 2 : 1), params);
}

function lucky_draw_shuffle(id, fade_steps, callback_steps, num_steps, initial_duration, main_callback, step, buffered) {
    if(!step) step = 0;
    if(!buffered) buffered = false;
    if(step == callback_steps) main_callback();
    if(step >= (fade_steps + num_steps)) return;

    $('promoted_buffer_' + id).show();
    $('promoted_reward_name_buffer_' + id).show();
    var duration = 0.025 + (step >= fade_steps ? 0 : (initial_duration - (step * (initial_duration / fade_steps))))

    var prev_opacity = (step >= fade_steps ? 0 : (initial_duration - (step * (initial_duration / fade_steps))))
    var next_opacity = (step >= fade_steps ? 0 : (initial_duration - (step * (initial_duration / fade_steps))))

    var callback = n_callback(4, function() {
        lucky_draw_shuffle(id, fade_steps, callback_steps, num_steps, initial_duration, main_callback, step + 1, !buffered);
    });

    new Effect.Opacity((buffered ? 'promoted_buffer_' : 'promoted_image_') + id, { from: 1.0, to: 0.0, duration: duration,
        afterFinish: callback });

    new Effect.Opacity((buffered ? 'promoted_image_' : 'promoted_buffer_') + id, { from: 0.0, to: 1.0, duration: duration,
        afterFinish: callback });

    new Effect.Opacity((buffered ? 'promoted_reward_name_buffer_' : 'promoted_reward_name_') + id, { from: 1.0, to: 0.0, duration: duration,
        afterFinish: callback });

    new Effect.Opacity((buffered ? 'promoted_reward_name_' : 'promoted_reward_name_buffer_') + id, { from: 0.0, to: 1.0, duration: duration,
        afterFinish: callback });
}

function lucky_draw_flip_cards(callback) {
    var fn = n_callback(2, callback);

    lucky_draw_flip_card(1, fn);
    lucky_draw_flip_card(2, fn);

    $('choose_message').show();
    $('choose_message').style['opacity'] = 1.0;
}

function lucky_draw_flip_card(id, callback) {
    $('promoted_reward_' + id).setStyle({ borderTopLeftRadius: '0px', borderTopRightRadius: '0px' });
    new Effect.Morph('promoted_reward_' + id, { style: { paddingTop: '13.75px', paddingBottom: '82.5px' }, duration: 0.5,
        transition: Effect.Transitions.linear, afterFinish: function() { $('promoted_reward_' + id).hide(); } });

    slide_names({ id: id, x: 5, y: 65, duration: 0.5 });

    $('reward_choice_slot_' + (id - 1)).setStyle({ height: "0px", opacity: 1.0 });
    $('reward_choice_slot_' + (id - 1)).show();
    new Effect.Morph('reward_choice_slot_' + (id - 1), { style: { height: '218px' }, duration: 0.6,
        transition: Effect.Transitions.linear, afterFinish: callback });
}

function lucky_draw_reveal_card(id, url, description, name) {
    $('promoted_buffer_2').hide();
    $('promoted_reward_name_buffer_2').hide();

    $('promoted_image_' + (id +1)).src = url;
    $('promoted_image_' + (id +1)).title = description;
    $('promoted_image_' + (id +1)).show();

    $('promoted_reward_name_' + (id +1)).innerHTML = name;
    $('promoted_reward_name_' + (id +1)).show();

    $('promoted_reward_name_1').setStyle({ top: '65px' });
    $('promoted_reward_name_2').setStyle({ top: '65px' });
    $('promoted_reward_name_buffer_2').setStyle({ top: '65px' });
    $('promoted_reward_name_buffer_1').setStyle({ top: '65px' });

    setTimeout(function() {
        slide_names({ id: 1, x: 5, y: 135, duration: 0.5 });
        slide_names({ id: 2, x: 5, y: 135, duration: 0.5 });

        $('promoted_reward_' + (id + 1)).show();
        $('promoted_reward_' + (id + 1)).setStyle({ borderRadius: '20px', paddingTop: '13.75px', paddingBottom: '82.5px' });
        new Effect.Morph('promoted_reward_' + (id + 1), { style: { paddingTop: '82.5px', paddingBottom: '82.5px' }, duration: 0.5,
            transition: Effect.Transitions.linear, afterFinish: function() {}
                                                        });
    }, 0.1);

    $('reward_choice_slot_' + id).setStyle({ height: "218px" });
    $('reward_choice_slot_' + id).show();
    new Effect.Morph('reward_choice_slot_' + id, { style: { height: '0px' }, duration: 0.6,
        transition: Effect.Transitions.linear, afterFinish: function() { $('reward_choice_slot_' + id).hide(); } });
}

function set_lucky_draw_rolled(val) {
    lucky_draw_rolled = val;

    if(lucky_draw_rolled) {
        $$('.reward_choice_container').each(function(elem) {
            elem.removeClassName('unrolled');
            elem.addClassName('rolled');
        });

    } else {
        $$('.reward_choice_container').each(function(elem) {
            elem.addClassName('unrolled');
            elem.removeClassName('rolled');
        });
    }
}

function lucky_draw_select_item(id, slot_id) {
    var slot = $('reward_' + slot_id);
    var pos = slot.positionedOffset();

    lucky_draw_move_selection(id, slot_id);

    $('selection_' + id).show();
    $('selection_' + id).setStyle({ top: (pos.top-9) + 'px', left: (pos.left-9) + 'px' });
}

function lucky_draw_clear_selected() {
    lucky_draw_selected = {};

    lucky_draw_update_selected();
}

function lucky_draw_move_selection(id, pos) {
    lucky_draw_selected[id] = pos;

    lucky_draw_update_selected();
}

function lucky_draw_update_selected() {
    var selected = $H(lucky_draw_selected);

    $$('#rewards .reward').each(function(e) {
        e.removeClassName('selected');

        if(selected.keys().length != 0) {
            e.addClassName('unselected');
        }
    });

    selected.each(function(pair) {
        $('reward_' + pair.value).removeClassName('unselected');
        $('reward_' + pair.value).addClassName('selected');
    });
}

function lucky_draw_disable_choice_buttons() {
    $$('.choice_button').each(function(item) {
        item.disabled = true;
    });
}

function n_callback(n, callback) {
    var count = 0;

    return function() {
        count += 1;
        if(count < n) return;

        callback();
    }
}

function update_token_count(n) {
    $('tokens').innerHTML = n.toString();
    if(n <= 0) { disable_element('roll_button'); }
}

function update_roll_button(n) {
    if(n > 0) { enable_element('roll_button'); }
}

function timed_update(which) {
    setTimeout(function() { update_lucky_grid(which); }, 2000);
}

function update_lucky_grid(which) {
    var callback = n_callback(10, hide_lucky_draw_elements);
    var duration = 0.5;

    new Effect.Opacity('promoted_reward_1', { from: 1.0, to: 0.0, duration: duration, afterFinish: callback });
    new Effect.Opacity('promoted_reward_2', { from: 1.0, to: 0.0, duration: duration, afterFinish: callback });

    new Effect.Opacity('reward_choice_slot_1', { from: 1.0, to: 0.0, duration: duration, afterFinish: callback });
    new Effect.Opacity('reward_choice_slot_0', { from: 1.0, to: 0.0, duration: duration, afterFinish: callback });

    if(which == 0) {
        new Effect.Opacity('promoted_image_1', { from: 1.0, to: 0.0, duration: duration, afterFinish: callback });
        new Effect.Opacity('promoted_buffer_1', { from: 0.0, to: 0.0, duration: duration, afterFinish: callback });
        new Effect.Opacity('promoted_image_2', { from: 1.0, to: 0.0, duration: duration, afterFinish: callback });
        new Effect.Opacity('promoted_buffer_2', { from: 0.0, to: 0.0, duration: duration, afterFinish: callback });

    } else {
        new Effect.Opacity('promoted_image_1', { from: 0.0, to: 0.0, duration: duration, afterFinish: callback });
        new Effect.Opacity('promoted_buffer_1', { from: 1.0, to: 0.0, duration: duration, afterFinish: callback });
        new Effect.Opacity('promoted_image_2', { from: 0.0, to: 0.0, duration: duration, afterFinish: callback });
        new Effect.Opacity('promoted_buffer_2', { from: 1.0, to: 0.0, duration: duration, afterFinish: callback });
    }

    new Effect.Opacity('choose_message', { from: 1.0, to: 0.0, duration: duration, afterFinish: callback  });
    appear_rewards(duration, callback);
}

function appear_rewards(duration, callback) {
    var rewards = $$('#rewards .reward');
    var rewards_callback = n_callback(rewards.length + 1, callback);

    var appear_callback = n_callback(rewards.length + 1, function() {
        $('rewards').show();

        rewards.each(function(reward) {
            new Effect.Opacity(reward, { from: 0.0, to: 1.0, duration: duration, afterFinish: rewards_callback });
        });

        new Effect.Opacity('rewards', { from: 0.0, to: 1.0, duration: duration, afterFinish: rewards_callback });
    });

    rewards.each(function(reward) {
        new Effect.Opacity(reward, { from: 0.0, to: 0.0, duration: 0, afterFinish: appear_callback });
    });

    new Effect.Opacity('rewards', { from: 0.0, to: 0.0, duration: 0, afterFinish: appear_callback });
}

function fade_rewards(duration, callback) {
    var rewards = $$('#rewards .reward');
    var rewards_callback = n_callback(rewards.length + 1, function() {
        $('rewards').hide();

        rewards.each(function(reward) {
            new Effect.Opacity(reward, { from: 1.0, to: 1.0, duration: 0 });
        });

        new Effect.Opacity('rewards', { from: 1.0, to: 1.0, duration: 0 });

        callback();
    });

    rewards.each(function(reward) {
        new Effect.Opacity(reward, { from: 1.0, to: 0.0, duration: duration, afterFinish: rewards_callback });
    });

    new Effect.Opacity('rewards', { from: 1.0, to: 0.0, duration: duration, afterFinish: rewards_callback });
}

function hide_lucky_draw_elements() {
    set_lucky_draw_visible();

    $('promoted_reward_1').hide();
    $('promoted_reward_2').hide();

    $('reward_choice_slot_1').hide();
    $('reward_choice_slot_0').hide();

    $('choose_message').hide();

}

function set_lucky_draw_visible() {
    new Effect.Opacity('promoted_reward_1', { from: 1.0, to: 1.0, duration: 0 });
    new Effect.Opacity('promoted_reward_2', { from: 1.0, to: 1.0, duration: 0 });

    new Effect.Opacity('reward_choice_slot_1', { from: 1.0, to: 1.0, duration: 0 });
    new Effect.Opacity('reward_choice_slot_0', { from: 1.0, to: 1.0, duration: 0 });

    new Effect.Opacity('promoted_image_1', { from: 1.0, to: 1.0, duration: 0 });
    new Effect.Opacity('promoted_buffer_1', { from: 0.0, to: 0.0, duration: 0 });
    new Effect.Opacity('promoted_image_2', { from: 1.0, to: 1.0, duration: 0 });
    new Effect.Opacity('promoted_buffer_2', { from: 0.0, to: 0.0, duration: 0 });

    new Effect.Opacity('choose_message', { from: 1.0, to: 1.0, duration: 0 });
}

function update_image_to_over(className) {
  i = document.getElementsByClassName(className)[0];
  source = i.src.replace('.png', '_over.png');
  i.setAttribute('src', source);
}

function revert_image(className) {
  i = document.getElementsByClassName(className)[0];
  source = i.src.replace('_over', '');
  i.setAttribute('src', source);
}

function loading_toggle_for(id) {
  $(id + '_form').toggle();
  $(id + '_loading').toggle();
}

skip_animation_flag = false;
function skip_lucky_draw_animation(item1, item2) {
  skip_animation_flag = true;
  lucky_draw_rolled = true;

  lucky_draw_promote_items(item1, item2);
  $$('.reward_choice_container').each(function(elem) {
      elem.removeClassName('unrolled');
      elem.addClassName('rolled');
  });
}

function update_lucky_draw_token_purcahse() {
  var amount = $F('amount');
  $('token_amount').innerHTML = amount;
  if (document.getElementById('normal_token_cost')) {
    $('normal_token_cost').innerHTML = $F('normal_price_per_token') * amount;
  }
  $('sale_token_cost').innerHTML = $F('sale_price_per_token') * amount;
}

function lucky_draw_save_for_later() {
    $('item_won_content').hide();
    $('save_for_later').show();
}

function hide_item_won() {
    if($('item_won')) $('item_won').hide();
}

function set_notification(id, msec) {
  setTimeout("show_notification("+ id + ")", msec);
}

function show_notification(id) {
  element = 'notification_popup_' + id
  $(element).show();
  new Effect.Morph(element, { style: { height: '40px', marginBottom: '1em', paddingTop: '1em', paddingBottom: '1em' },
    duration: 0.5, transition: Effect.Transitions.linear });
}

function remove_notification(id) {
  element = 'notification_popup_' + id
  new Effect.Morph(element, { style: { height: '0px', marginBottom: '0em', paddingTop: '0em', paddingBottom: '0em' }, duration: 0.5,
      transition: Effect.Transitions.linear, afterFinish: function() {$(element).hide();} });
}

function submitOnEnter(event, text_field, button_name) {
  if (event.keyCode == 13) {
    $(button_name).click();
    text_field.blur();
  }
}

function change_selected_planet(drop_down) {
  var atts = drop_down.options[drop_down.selectedIndex].attributes;
  var image_string = atts['tag'].value;
  $('planet_dropdown_selected_image').src = image_string;

  $('galaxy').value = atts['galaxy'].value;
  $('solar_system').value = atts['solar_system'].value;
  $('planet').value = atts['planet'].value;
  $('planet_type').value = atts['type'].value;

}

function remove_store_popup(popup_id) {
  $('content').removeChild($(popup_id));
  $('content').removeChild($('store_popup_overlay'));

  $$('.disable_link_overlay').each(function(elem) {
    elem.hide();
  });

  enable_ajax_links();
}

function remove_popup(popup, overlay) {
  $('content').removeChild($(popup));
  $('content').removeChild($(overlay));
}

function disable_popup_cancel() {
  $('store_popup_overlay').onclick = '';
  var close_button = $('store_popup_close')
  if (close_button != null)
    close_button.onclick = '';
}

function disable_store_links() {
  $$('.disable_link_overlay').each(function(elem) {
    elem.show();
  });
}

function update_store_button_text(normal_price, sale_price, text_box) {
  $$('.button_price').each(function(elem) {
    elem.innerHTML = format_number(normal_price * parseInt(text_box.value));
  });
  if(sale_price != '')
    $$('.button_sale_price').each(function(elem) {
      elem.innerHTML = format_number(sale_price * parseInt(text_box.value));
    });
}

function get_achievement_perm() {
  FB.login(function(response) { }, {scope:'publish_actions'});
}

function post_to_wall(wall_post) {
  wall_post = wall_post.evalJSON();
  wall_post['method'] = 'stream.publish';

  FB.ui(wall_post,
    function(response) {
      // This can be used for a callback if the post was made or not
      // if (response && response.post_id) {
      //    alert('Post was published.');
      //  } else {
      //    alert('Post was not published.');
      //  }
    }
  );
}

var chat_scroll_lock = false;
function set_chat_scroll_lock(val) {
    chat_scroll_lock = val;

    val ? $('botfade').hide() : $('botfade').show();
}

var chat_socket;
function init_chat() {
    update_chat_shadows();

    chat_scroll_to_bottom();
    set_chat_scroll_lock(true);

    $('chat_messages').cleanWhitespace();
}

function open_chat_socket(host, port, user_id, auth_token, name) {
    if(chat_socket) return;

    var Socket = "MozWebSocket" in window ? MozWebSocket : WebSocket;
    chat_socket = new Socket("ws://" + host + ":" + port + "/");

    chat_socket.onmessage = function(evt) {
        data = evt.data.evalJSON(true);
        message = structured_message(data);

        chat(data['type'], message);
    };

    chat_socket.onclose = function() { chat('important', '*** Disconnected from chat.'); }
    chat_socket.onopen = function() {
        chat_socket.send("/authenticate " + user_id + " " + auth_token + " " + name);
    };
}

function update_chat_shadows() {
    var scrollbar_width = get_scrollbar_width();
    var messages_width = $('chat_messages').getWidth();

    var width = (messages_width - scrollbar_width - 2);

    $('topfade').setStyle({ width: width + 'px' });
    $('botfade').setStyle({ width: width + 'px' });
}

function chat_scroll_to_bottom() {
    $('chat_messages').scrollTop = $('chat_messages').scrollHeight;
}

function on_chat_scroll() {
    var pos = $('chat_messages').getHeight() + $('chat_messages').scrollTop;
    var max = $('chat_messages').scrollHeight;

    set_chat_scroll_lock(pos == max);
}

function get_scrollbar_width() {
    var inner = new Element('p').setStyle({ width: '100%', height: '200px' });
    var outer = new Element('div').setStyle({ position: 'absolute', top: '0px', left: '0px', visibility: 'hidden', width: '200px',
      height: '150px', overflow: 'hidden' });
    outer.insert({ bottom: inner });

    $(document.body).insert({ bottom: outer });
    var w1 = inner.offsetWidth;

    outer.style.overflow = 'scroll';
    var w2 = inner.offsetWidth;
    if(w1 == w2) w2 = outer.clientWidth;

    outer.remove();

    return w1 - w2;
}

function chat(type, message) {
    var elem = new Element('div', { 'class': 'message ' + type }).update(message);

    $('botfade').insert({ before: elem });

    while($('chat_messages').childElements().length > 100) {
        var to_remove = $('topfade').nextSibling;
        var height = to_remove.getHeight();

        to_remove.remove();
        $('chat_messages').scrollTop -= height;
    }

    if(chat_scroll_lock) chat_scroll_to_bottom();
}

function send_message() {
    var message = $F('chat_reply');
    $('chat_reply').value = "";

    if(message.length <= 200) {
        chat_socket.send(message);

    } else {
        chat('error', "Message is too long (max 200 characters).");
    }
}

function structured_message(data) {
    var message = data['message']
    var type = data['type']
    var sender = data['sender']
    var target = data['target']

    switch(type) {
    case 'alliance': message = "[Alliance] [" + sender + "]: " + message; break;
    case 'whisper': message = "[" + sender + "] whispers: " + message; break;
    case 'chat': message = "[" + sender + "] says: " + message; break;
    case 'action': message = "* " + sender + " " + message; break;
    case 'notice': break;
    case 'important': break;
    case 'error': break;
    default:
        console.log('Unrecognized message type: ' + type);
    }

    return message;
}

function random_element(arry) {
    return arry[Math.floor(Math.random() * arry.length)];
}

function verfiy_no_commas_and_submit(text_field_id, submit_id, error_id, link) {
  if ($(text_field_id).value.indexOf(',') == -1) {
    $(submit_id).click();
    link.writeAttribute('onclick', 'return false;');
    $(error_id).style['display'] = 'none';
  } else {
    $(error_id).innerHTML = 'Commas not allowed'
    $(error_id).style['display'] = 'block';
  }
}
