#!/usr/bin/perl

use strict;
use warnings qw{ all };

use SFC::Fleet;
use SFC::SimpleSim qw{ simulate };

my $attacker = new SFC::Fleet;
my $defender = new SFC::Fleet;

$attacker->get_attacker_options('pass_through');
$defender->get_defender_options('pass_through');

my $rounds = simulate($attacker, $defender, sub {
        my ($r, $a, $d) = @_;

        print "End of round $r\n\n";
        print "Attacker's ships and defenses:\n\n";
        $a->print_status();
        print "Defender's ships and defenses:\n\n";
        $d->print_status();
    });

if ($attacker->destroyed()) {
    if ($defender->destroyed()) {
        print "Draw, both sides destroyed.\n";
    }
    else {
        print "Defender wins with\n* ";
        print join("\n* ", $defender->summary_str()), "\n";
    }
}
else {
    if ($defender->destroyed()) {
        print "Attacker wins with\n* ";
        print join("\n* ", $attacker->summary_str()), "\n";
    }
    else {
        print "Draw,\n";
        print "Attacker:\n* ", join("\n* ", $attacker->summary_str()), "\n";
        print "Defender:\n* ", join("\n* ", $defender->summary_str()), "\n";
    }
}


print "after $rounds rounds.\n";

printf "Attacker losses: %d ore, %d crystal, %d hydrogen\n",
    $attacker->ore_lost(), $attacker->crystal_lost(),
    $attacker->hydrogen_lost();

printf "Defender losses: %d ore, %d crystal, %d hydrogen\n",
    $defender->ore_lost(), $defender->crystal_lost(),
    $defender->hydrogen_lost();

print "Total debris:  ", $attacker->debris() + $defender->debris(), "\n";

print "Attacker overkill taken: ", $attacker->overkill_taken(), "\n";
print "Defender overkill taken: ", $defender->overkill_taken(), "\n";
