#!/usr/bin/perl

use strict;
use warnings qw{ all };

use Statistics::Descriptive;

use SFC::Fleet;
use SFC::SimpleSim qw{ simulate };

my $attacker = new SFC::Fleet;
my $defender = new SFC::Fleet;

$attacker->get_attacker_options('pass_through');
$defender->get_defender_options('pass_through');


my $win_index = Statistics::Descriptive::Full->new();
my $att_overkill_taken = Statistics::Descriptive::Full->new();
my $def_overkill_taken = Statistics::Descriptive::Full->new();
my $debris = Statistics::Descriptive::Full->new();
my $att_ore_lost = Statistics::Descriptive::Full->new();
my $att_cry_lost = Statistics::Descriptive::Full->new();
my $att_gen_lost = Statistics::Descriptive::Full->new();
my $def_ore_lost = Statistics::Descriptive::Full->new();
my $def_cry_lost = Statistics::Descriptive::Full->new();
my $def_gen_lost = Statistics::Descriptive::Full->new();

for (my $i = 50; $i > 0; --$i) {
    my $att = $attacker->clone();
    my $def = $defender->clone();

    my $rounds = simulate($att, $def, sub { });
    $win_index->add_data($att->destroyed() ? 
            ($def->destroyed() ? 0 : -7 + $rounds) :
            ($def->destroyed() ? 7 - $rounds : 0));
    $att_overkill_taken->add_data($att->overkill_taken());
    $def_overkill_taken->add_data($def->overkill_taken());
    $debris->add_data($att->debris() + $def->debris());
    $att_ore_lost->add_data($att->ore_lost());
    $att_cry_lost->add_data($att->crystal_lost());
    $att_gen_lost->add_data($att->hydrogen_lost());
    $def_ore_lost->add_data($def->ore_lost());
    $def_cry_lost->add_data($def->crystal_lost());
    $def_gen_lost->add_data($def->hydrogen_lost());
}

printf "Attack success index:  %.2f±%.2f\n", $win_index->mean,
        $win_index->standard_deviation;

#        print "Attacker:\n* ", join("\n* ", $attacker->summary_str()), "\n";
#        print "Defender:\n* ", join("\n* ", $defender->summary_str()), "\n";

printf "Attacker losses:  %d±%d ore, %d±%d crystal, %d±%d hydrogen\n",
        $att_ore_lost->mean(), $att_ore_lost->standard_deviation(),
        $att_cry_lost->mean(), $att_cry_lost->standard_deviation(),
        $att_gen_lost->mean(), $att_gen_lost->standard_deviation();

printf "Defender losses:  %d±%d ore, %d±%d crystal, %d±%d hydrogen\n",
        $def_ore_lost->mean(), $def_ore_lost->standard_deviation(),
        $def_cry_lost->mean(), $def_cry_lost->standard_deviation(),
        $def_gen_lost->mean(), $def_gen_lost->standard_deviation();

printf "Total debris:  %d±%d\n", $debris->mean(), $debris->standard_deviation();

printf "Attacker overkill taken:  %d±%d\n", $att_overkill_taken->mean(),
        $att_overkill_taken->standard_deviation();
printf "Defender overkill taken:  %d±%d\n", $def_overkill_taken->mean(),
        $def_overkill_taken->standard_deviation();
