#!/usr/bin/perl

use utf8;
# use locale;
binmode STDOUT, ':utf8';
binmode STDIN,  ':utf8';

use strict;
use warnings qw{ all };

use Statistics::Descriptive;

use SFC::FleetSummary;
use SFC::SimpleSim qw{ simulate };


my $attacker = new SFC::FleetSummary;
my $defender = new SFC::FleetSummary;


#
# Get options
#
my $sims = 100;

$attacker->get_attacker_options('pass_through');
$defender->get_defender_options('pass_through');


#
# Initialize statistics
#
my @STATS = qw{ count overkill_given overkill_taken ships_lost };

my $win_index = Statistics::Descriptive::Full->new();
my $att_overkill_taken = Statistics::Descriptive::Full->new();
my $def_overkill_taken = Statistics::Descriptive::Full->new();
my $debris = Statistics::Descriptive::Full->new();
my $att_ore_lost = Statistics::Descriptive::Full->new();
my $att_cry_lost = Statistics::Descriptive::Full->new();
my $att_gen_lost = Statistics::Descriptive::Full->new();
my $def_ore_lost = Statistics::Descriptive::Full->new();
my $def_cry_lost = Statistics::Descriptive::Full->new();
my $def_gen_lost = Statistics::Descriptive::Full->new();

my %att_stats;
for my $t ($attacker->types()) {
    for my $k (@STATS) {
        $att_stats{$t}{$k} = Statistics::Descriptive::Full->new();
    }
}

my %def_stats;
for my $t ($defender->types()) {
    for my $k (@STATS) {
        $def_stats{$t}{$k} = Statistics::Descriptive::Full->new();
    }
}

#
# Main loop
#
for (my $i = $sims; $i > 0; --$i) {
    my $att = $attacker->instance();
    my $def = $defender->instance();

    my $rounds = simulate($att, $def, sub { });
    $win_index->add_data($att->destroyed() ? 
            ($def->destroyed() ? 0 : -7 + $rounds) :
            ($def->destroyed() ? 7 - $rounds : 0));
    # $att_overkill_taken->add_data($att->overkill_taken());
    # $def_overkill_taken->add_data($def->overkill_taken());
    $debris->add_data($att->debris() + $def->debris());
    $att_ore_lost->add_data($att->ore_lost());
    $att_cry_lost->add_data($att->crystal_lost());
    $att_gen_lost->add_data($att->hydrogen_lost());
    $def_ore_lost->add_data($def->ore_lost());
    $def_cry_lost->add_data($def->crystal_lost());
    $def_gen_lost->add_data($def->hydrogen_lost());
    for my $t ($attacker->types()) {
        for my $k (@STATS) {
            $att_stats{$t}{$k}->add_data($att->results()->{$t}{$k});
        }
    }
    for my $t ($defender->types()) {
        for my $k (@STATS) {
            $def_stats{$t}{$k}->add_data($def->results()->{$t}{$k});
        }
    }
}

#
# Print results
#
printf "Attack success index:  %.2f±%.2f\n", $win_index->mean,
        $win_index->standard_deviation;

#        print "Attacker:\n* ", join("\n* ", $attacker->summary_str()), "\n";
#        print "Defender:\n* ", join("\n* ", $defender->summary_str()), "\n";

printf "Attacker losses:  %d±%d ore, %d±%d crystal, %d±%d hydrogen\n",
        $att_ore_lost->mean(), $att_ore_lost->standard_deviation(),
        $att_cry_lost->mean(), $att_cry_lost->standard_deviation(),
        $att_gen_lost->mean(), $att_gen_lost->standard_deviation();

printf "Defender losses:  %d±%d ore, %d±%d crystal, %d±%d hydrogen\n",
        $def_ore_lost->mean(), $def_ore_lost->standard_deviation(),
        $def_cry_lost->mean(), $def_cry_lost->standard_deviation(),
        $def_gen_lost->mean(), $def_gen_lost->standard_deviation();

printf "Total debris:  %d±%d\n", $debris->mean(), $debris->standard_deviation();

# printf "Attacker overkill taken:  %d±%d\n", $att_overkill_taken->mean(),
#         $att_overkill_taken->standard_deviation();
# printf "Defender overkill taken:  %d±%d\n", $def_overkill_taken->mean(),
#         $def_overkill_taken->standard_deviation();

print "\nATTACKING FLEET\n";
for my $t ($attacker->types()) {
    print $t, ":  ";
    for my $k (@STATS) {
        print $k, "=";
        printf "%.1f±%.1f", $att_stats{$t}{$k}->mean(), 
            $att_stats{$t}{$k}->standard_deviation(), 
    }
    continue {
        print ", ";
    }
    print "\n";
}

print "\nDEFENDING FLEET\n";
for my $t ($defender->types()) {
    print $t, ":  ";
    for my $k (@STATS) {
        print $k, "=";
        printf "%.1f±%.1f", $def_stats{$t}{$k}->mean(), 
            $def_stats{$t}{$k}->standard_deviation(), 
    }
    continue {
        print ", ";
    }
    print "\n";
}
