#!/usr/bin/perl

use utf8;
# use locale;

binmode STDOUT, ':utf8';
binmode STDIN,  ':utf8';

use strict;
use warnings qw{ all };

use Getopt::Long;
use IO::Select;

use SFC::FleetSummary;
use SFC::Sim;
use SFC::Statistics;



my $attackers = []; #  SFC::FleetSummary;
my $defenders = []; #  SFC::FleetSummary;

my $ore_plunder;
my $cry_plunder;
my $gen_plunder;
my $target;

my $debris;

sub read_fsf {
    my ($name, $value) = @_;

    my $fs = new SFC::FleetSummary;
    my $is_target = $name eq 'def';
    $fs->read_file($value, $is_target);

    if ($name eq 'att' || $name eq 'ga') {
        push @$attackers, $fs;
    }
    elsif ($name eq 'gd') {
        push @$defenders, $fs;
    }
    elsif ($is_target) {
        push @$defenders, $fs;
        $ore_plunder ||= $fs->ore_plunder();
        $cry_plunder ||= $fs->crystal_plunder();
        $gen_plunder ||= $fs->hydrogen_plunder();
        $target ||= $fs->location();
    }
    else {
        die "read_fsf can't happen error";
    }
}

my $sims = 100;
my ($dump, $prof);

GetOptions( 'att=s@'  => \&read_fsf, 
            'def=s@'  => \&read_fsf,
            'ga=s@'   => \&read_fsf,
            'gd=s@'   => \&read_fsf,
            'num=i'   => \$sims,
            'dump!'   => \$dump,
            'debris!' => \$debris,
            'prof!'   => \$prof,
    ) or die 'Error:  parsing options';

if (! defined $debris) {
    $debris = $target->type ne 'e';
}

my $stats = new SFC::Statistics($attackers, $defenders,
        $ore_plunder, $cry_plunder, $gen_plunder, $target, $debris);

my @sim;


if ($prof) {
    $sim[0] = SFC::Sim->new('ssh', 'cephas',
        'cd proj/al/sfc; nice perl -d:DProf ./simserv.pl');
}
else {
    $sim[0] = SFC::Sim->new('nice', './simserv.pl');
    $sim[1] = SFC::Sim->new('ssh', 'cephas',
            'cd proj/al/sfc; nice ./simserv.pl');
    $sim[2] = SFC::Sim->new('nice', './simserv.pl');
    $sim[3] = SFC::Sim->new('ssh', 'cephas',
            'cd proj/al/sfc; nice ./simserv.pl');
#    $sim[4] = SFC::Sim->new('ssh', 'duns-scotus',
#            'cd proj/al/sfc; nice ./simserv.pl');
}

if ($dump) {
    $sim[0]->dump($attackers, $defenders);
    exit;
}

run_sims($sims, $stats, @sim);

$stats->print_stats;


sub run_sims {
    my ($left_to_start, $stats, @sim) = @_;
    for my $s (@sim) {
        $s->init($attackers, $defenders);
    }

    my $sel = new IO::Select;
    my %sim;

    for my $s (@sim) {
        my $fd = $s->read_fileno;
        $s->start;
        $sel->add($fd);
        $sim{$fd} = $s;
        --$left_to_start or last;
    }

    while (my @ready = $sel->can_read) {
        for my $fd (@ready) {
            my $s = $sim{$fd};
            $stats->record_data($s->collect);

            if ($left_to_start-- > 0) {
                $s->start;
            }
            else {
                $sel->remove($fd);
            }
        }

        last if ! $sel->count;
    }
}

