#!/usr/bin/perl

use utf8;
# use locale;
binmode STDOUT, ':utf8';
binmode STDIN,  ':utf8';

use strict;
use warnings qw{ all };

use Statistics::Descriptive;

use SFC::FleetSummary;
use SFC::MultiSim;


my $attacker = new SFC::FleetSummary;
my $defender = new SFC::FleetSummary;


#
# Get options
#
my $sims = 100;

$attacker->get_attacker_options('pass_through');
$defender->get_defender_options('pass_through');


#
# Create simulation
#

my $sim = SFC::MultiSim->new($sims, $attacker, $defender);


#
# Print results
#
printf "Attack success index:  %.2f±%.2f\n", $sim->win_index->mean,
        $sim->win_index->standard_deviation;

printf "Attacker losses:  %d±%d ore, %d±%d crystal, %d±%d hydrogen\n",
        $s->att_ore_lost->mean(), $s->att_ore_lost->standard_deviation(),
        $s->att_cry_lost->mean(), $s->att_cry_lost->standard_deviation(),
        $s->att_gen_lost->mean(), $s->att_gen_lost->standard_deviation();

printf "Defender losses:  %d±%d ore, %d±%d crystal, %d±%d hydrogen\n",
        $s->def_ore_lost->mean(), $s->def_ore_lost->standard_deviation(),
        $s->def_cry_lost->mean(), $s->def_cry_lost->standard_deviation(),
        $s->def_gen_lost->mean(), $s->def_gen_lost->standard_deviation();

printf "Total debris:  %d±%d\n",
        $s->debris->mean(), $s->debris->standard_deviation();

print "\nATTACKING FLEET\n";
for my $t ($attacker->types()) {
    print $t, ":  ";
    for my $k (@STATS) {
        print $k, "=";
        printf "%.1f±%.1f", $s->att_stats{$t}{$k}->mean(), 
            $s->att_stats{$t}{$k}->standard_deviation(), 
    }
    continue {
        print ", ";
    }
    print "\n";
}

print "\nDEFENDING FLEET\n";
for my $t ($defender->types()) {
    print $t, ":  ";
    for my $k (@STATS) {
        print $k, "=";
        printf "%.1f±%.1f", $s->def_stats{$t}{$k}->mean(), 
            $s->def_stats{$t}{$k}->standard_deviation(), 
    }
    continue {
        print ", ";
    }
    print "\n";
}
