#include <json-glib/json-glib.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>

#include "simserv.h"

enum { NA, JET, PULSE, WARP };

struct ship_stat {
    char *tag, *fullname;
    short ship, engine;
    int ore, cry, gen;
    int cargo, speed, fuel;
    int weapon, shield, hull;
    int sorto;
} base_stats[STZ] = {
    {
        "Mis", "Missile Battery",
        0, NA,
        2000, 0, 0,
        0, 0, 0,
        80, 20, 200,
        21,
    }, {
        "Las", "Laser Cannon",
        0, NA,
        1500, 500, 0,
        0, 0, 0,
        100, 25, 200,
        22,
    }, {
        "Pul", "Pulse Cannon",
        0, NA,
        6000, 2000, 0,
        0, 0, 0,
        250, 100, 800,
        23,
    }, {
        "Par", "Particle Cannon",
        0, NA,
        2000, 6000, 0,
        0, 0, 0,
        150, 500, 800,
        24,
    }, {
        "Dec", "Decoy",
        0, NA,
        10000, 10000, 0,
        0, 0, 0,
        0, 2000, 2000,
        25,
    }, {
        "Gau", "Gauss Cannon",
        0, NA,
        20000, 15000, 2000,
        0, 0, 0,
        1100, 200, 3500,
        26,
    }, {
        "Lar", "Large Decoy",
        0, NA,
        50000, 50000, 0,
        0, 0, 0,
        0, 10000, 10000,
        27,
    }, {
        "Pla", "Plasma Cannon",
        0, NA,
        50000, 50000, 30000,
        0, 0, 0,
        3000, 300, 10000,
        28,
    }, {
        "Hrm", "Hermes Class Probe",
        1, JET,
        0, 1000, 0,
        5, 100000000, 1,
        0, 0, 100,
        1,
    }, {
        "Hel", "Helios Class Solar Satellite",
        1, NA,
        0, 2000, 500,
        0, 0, 0,
        1, 1, 200,
        2,
    }, {
        "Art", "Artemis Class Fighter",
        1, JET,
        3000, 1000, 0,
        50, 12500, 20,
        50, 10, 400,
        3,
    }, {
        "Atl", "Atlas Class Cargo",
        1, PULSE,
        2000, 2000, 0,
        5000, 10000, 20,       /* upgraded from 5000, 10 */
        5, 10, 400,
        4,
    }, {
        "Apo", "Apollo Class Fighter",
        1, PULSE,
        6000, 2500, 0,
        100, 10000, 75,
        150, 25, 850,
        5,
    }, {
        "Cha", "Charon Class Transport",
        1, PULSE,
        4000, 4000, 1000,
        100, 10000, 20,
        1, 25, 800,
        6,
    }, {
        "Hrc", "Hercules Class Cargo",
        1, JET,
        6000, 6000, 0,
        25000, 7500, 50,
        5, 25, 1200,
        7,
    }, {
        "Dio", "Dionysus Class Recycler",
        1, JET,
        10000, 6000, 2000,
        20000, 2000, 300,
        1, 10, 1600,
        8,
    }, {
        "Pos", "Posidon Class Cruiser",
        1, PULSE,
        20000, 7000, 2000,
        800, 15000, 300,
        400, 50, 2700,
        9,
    }, {
        "Gai", "Gaia Class Colony Ship",
        1, PULSE,
        10000, 20000, 10000,
        7500, 2500, 1000,
        50, 100, 3000,
        10,
    }, {
        "Ath", "Athena Class Battleship",
        1, WARP,
        45000, 15000, 0,
        1500, 10000, 500,
        1000, 200, 6000,
        11,
    }, {
        "Are", "Ares Class Bomber",
        1, WARP,
        50000, 25000, 15000,
        500, 5000, 1000,
        1000, 500, 7500,
        12,
    }, {
        "Had", "Hades Class Battleship",
        1, WARP,
        30000, 40000, 15000,
        750, 10000, 250,
        700, 400, 7000,
        13,
    }, {
        "Pro", "Prometheus Class Destroyer",
        1, WARP,
        60000, 50000, 15000,
        2000, 5000, 1000,
        2000, 500, 11000,
        14,
    }, {
        "Zeu", "Zeus Class",
        1, WARP,
        5000000, 4000000, 1000000,
        1000000, 100, 1,
        200000, 50000, 900000,
        15,
    }, {
        "Hep", "Hephaestus Class Attack Platform",
        1, WARP,
        20000000, 20000000, 10000000,
        1000000000, 90, 450000,
        0, 150000, 4000000,
        16,
    },
};



static JsonParser *parser;
static GError **error;
static JsonGenerator *generator;


int
jfs_count_ships(JsonObject *jfs)
{
    int i;
    int s = 0;
    JsonObject *jship;

    for (i = MIS; i < STZ; ++i) {
        if (json_object_has_member(jfs, base_stats[i].tag)) {
            jship = json_object_get_object_member(jfs, base_stats[i].tag);
            s += json_object_get_int_member(jship, "count");
        }
    }
    return s;
}


Ship *
jfs_write_ships(JsonObject *jfs, int fleet_id, Ship *s)
{
    int i, sc;
    int arm, wea, shi;
    int hl, at, sh;
    JsonObject *jship;
    
    arm = json_object_get_int_member(jfs, "armor");
    wea = json_object_get_int_member(jfs, "weapon");
    shi = json_object_get_int_member(jfs, "shield");

    for (i = MIS; i < STZ; ++i) {
        if (json_object_has_member(jfs, base_stats[i].tag)) {
            jship = json_object_get_object_member(jfs, base_stats[i].tag);
            sc = json_object_get_int_member(jship, "count");
            if (sc <= 0) continue;
            hl = base_stats[i].hull * (1.0 + 0.1 * arm);
            at = base_stats[i].weapon * (1.0 + 0.1 * wea);
            sh = base_stats[i].shield * (1.0 + 0.1 * shi);
            for ( ; sc--; ++s) {
                s->type = i;
                s->fleet_id = fleet_id;
                s->weapon = at;
                s->max_shield = sh;
                s->max_hull = hl;
            }
        }
    }
    return s;
}


void
init_json(void)
{
    g_type_init();
    parser = json_parser_new();
    generator = json_generator_new();
    // g_object_unref(parser);
}


void
fleet_from_json(Fleet *f, char *json)
{
    int ships = 0, fleets = 0;
    int i;
    JsonArray *jfsa;
    JsonObject *jfs;
    Ship *ship;
    JsonNode *root_node;

    fleet_free(f);
    json_parser_load_from_data(parser, json, strlen(json), error);
    root_node = json_parser_get_root(parser);
    f->json = (void *)root_node;
    jfsa = json_node_get_array(root_node);
    fleets = json_array_get_length(jfsa);       /* count fleets */
    for (i = 0; i < fleets; ++i) {
        jfs = json_array_get_object_element(jfsa, i);
        ships += jfs_count_ships(jfs);
    }
    fleet_alloc(f, fleets, ships);
    ship = f->data;
    for (i = 0; i < fleets; ++i) {              /* populate *f */
        jfs = json_array_get_object_element(jfsa, i);
        ship = jfs_write_ships(jfs, i, ship);
    }
}


char *
json_from_fleet(Fleet *f, char *buf)
{
    gchar *data;
    gsize len;
    JsonNode *root_node;
    JsonArray *jfsa;
    JsonObject *jfs;
    int i;

fprintf(stderr, "1\n");
    root_node = (JsonNode *)f->json;
fprintf(stderr, "2\n");
    jfsa = json_node_get_array(root_node);

fprintf(stderr, "3\n");
    for (i = 0; i < f->fleets; ++i) {
        jfs = json_array_get_object_element(jfsa, i);
    }

fprintf(stderr, "4\n");

    json_generator_set_root(generator, root_node);
fprintf(stderr, "5\n");
    data = json_generator_to_data(generator, &len);
    fprintf(stderr, "len = %d\n", (int)len);
    if (len > BUF_SIZE) {
        fprintf(stderr, "Generated JSON too large (%d)\n", (int)len);
        exit(1);
    }
    fprintf(stderr, "old buf: ->%s<-\n", buf);
    fprintf(stderr, "Data: ->%s<-\n", data);
    strcpy(buf, data);
    g_free(data);

    return buf;
}

