#include <stdlib.h>
#include <stdio.h>

#include "simserv.h"


#define	rand	((double) random() / (double) RAND_MAX)

/*
 * rapid fire table
 */

#define rfv2f(v) (1.0 - 1.0 / v)

float rapidfire[STZ][STZ] = {
    /* Mis 0 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
    /* Las 1 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
    /* Pul 2 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
    /* Par 3 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
    /* Dec 4 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
    /* Gau 5 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
    /* Lar 6 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
    /* Pla 7 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
    /* Hrm 8 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
    /* Hel 9 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
    /* Art 10 */
    { 0, 0, 0, 0, 0, 0, 0, 0,
	0.8, 0.8, 0, 0, 0, 0.8, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
    /* Atl 11 */
    { 0, 0, 0, 0, 0, 0, 0, 0,
	0.8, 0.8, 0, 0, 0, 0.8, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
    /* Apo 12 */
    { 0, 0, 0, 0, 0, 0, 0, 0,
	0.8, 0.8, 0, rfv2f(3), 0, 0.8, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
    /* Cha 13 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
    /* Hrc 14 */
    { 0, 0, 0, 0, 0, 0, 0, 0,
	0.8, 0.8, 0, 0, 0, 0.8, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
    /* Dio 15 */
    { 0, 0, 0, 0, 0, 0, 0, 0,
	0.8, 0.8, 0, 0, 0, 0.8, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
    /* Pos 16 */
    { 0.9, 0, 0, 0, 0, 0, 0, 0,
	0.8, 0.8, rfv2f(6), 0, 0, 0.8, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
    /* Gai 17 */
    { 0, 0, 0, 0, 0, 0, 0, 0,
	0.8, 0.8, 0, 0, 0, 0.8, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
    /* Ath 18 */
    { 0, 0, 0, 0, 0, 0, 0, 0,
	0.8, 0.8, 0, 0, 0, 0.8, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
    /* Are 19 */
    { 0.95, 0.95, 0.9, 0.9, 0, 0, 0, 0,
	0.8, 0.8, 0, 0, 0, 0.8, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
    /* Had 20 */
    { 0, 0, 0, 0, 0, 0, 0, 0,
	0.8, 0.8, 0, rfv2f(3), 0.75, 0.8, rfv2f(3), 0,
	0.75, 0, rfv2f(7), 0, 0, 0, 0, 0 },
    /* Pro 21 */
    { 0, 0.9, 0, 0, 0, 0, 0, 0,
	0.8, 0.8, 0, 0, 0, 0.8, 0, 0, 0, 0, 0, 0, 0.5, 0, 0, 0 },
    /* Zeu 22 */
    { 0.995, 0.995, 0.99, 0.99, 0, 0.98, 0, 0,
        0.9992, 0.9992, 0.995, 0.996, 0.99, 0.9992, 0.996, 0.996,
        rfv2f(33), 0.996, rfv2f(30), 0.96, rfv2f(15), 0.8, 0, 0 },
    /* Hep 23 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 }
};

#define rapidfire_succeeds(a, d) (rapidfire[a->type][d->type] == 0 ? \
	 0 : (rand > rapidfire[a->type][d->type]))

/*
 * ineffectiveness rule table
 */

#define MIS_BIT 0x00000001
#define LAS_BIT 0x00000002
#define PUL_BIT 0x00000004
#define PAR_BIT 0x00000008
#define DEC_BIT 0x00000010
#define GAU_BIT 0x00000020
#define LAR_BIT 0x00000040
#define PLA_BIT 0x00000080

#define HRM_BIT 0x00000100
#define HEL_BIT 0x00000200
#define ART_BIT 0x00000400
#define ATL_BIT 0x00000800
#define APO_BIT 0x00001000
#define CHA_BIT 0x00002000
#define HRC_BIT 0x00004000
#define DIO_BIT 0x00008000

#define POS_BIT 0x00010000
#define GAI_BIT 0x00020000
#define ATH_BIT 0x00040000
#define ARE_BIT 0x00080000
#define HAD_BIT 0x00100000
#define PRO_BIT 0x00200000
#define ZEU_BIT 0x00400000
#define HEP_BIT 0x00800000

unsigned int ineffective_tab[STZ] = {  /* ship is ineffective against ... */
    /* Mis  0 */ LAR_BIT | ZEU_BIT | HEP_BIT,
    /* Las  1 */ ZEU_BIT | HEP_BIT,
    /* Pul  2 */ ZEU_BIT | HEP_BIT,
    /* Par  3 */ ZEU_BIT | HEP_BIT,
    /* Dec  4 */ 0xFFFFFFFF,
    /* Gau  5 */ HEP_BIT,
    /* Lar  6 */ 0xFFFFFFFF,
    /* Pla  7 */ 0,
    /* Hrm  8 */ 0xFFFFFFFF,
    /* Hel  9 */ PUL_BIT | PAR_BIT | DEC_BIT | LAR_BIT |
                ATH_BIT | ARE_BIT | HAD_BIT | PRO_BIT | ZEU_BIT | HEP_BIT,
    /* Art 10 */ LAR_BIT | ZEU_BIT | HEP_BIT,
    /* Atl 11 */ PAR_BIT | DEC_BIT | LAR_BIT |
                ARE_BIT | PRO_BIT | ZEU_BIT | HEP_BIT,
    /* Apo 12 */ ZEU_BIT | HEP_BIT,
    /* Cha 13 */ PUL_BIT | PAR_BIT | DEC_BIT | LAR_BIT |
                GAI_BIT | ATH_BIT | ARE_BIT | HAD_BIT |
                PRO_BIT | ZEU_BIT | HEP_BIT,
    /* Hrc 14 */ PAR_BIT | DEC_BIT | LAR_BIT |
                ARE_BIT | PRO_BIT | ZEU_BIT | HEP_BIT,
    /* Dio 15 */ PUL_BIT | PAR_BIT | DEC_BIT | LAR_BIT |
                GAI_BIT | ATH_BIT | ARE_BIT | HAD_BIT |
                PRO_BIT | ZEU_BIT | HEP_BIT,
    /* Pos 16 */ ZEU_BIT | HEP_BIT,
    /* Gai 17 */ LAR_BIT | ZEU_BIT | HEP_BIT,
    /* Ath 18 */ HEP_BIT,
    /* Are 19 */ HEP_BIT,
    /* Had 20 */ HEP_BIT,
    /* Pro 21 */ 0,
    /* Zeu 22 */ 0,
    /* Hep 23 */ 0xFFFFFFFF,
};

#define ineffective(a, d) (ineffective_tab[a->type] & 1 << d->type)


/*
 * attack
 */

void
attack(Fleet *a, Fleet *d)
{
    int i = a->count;
    int range = d->count;
    Ship *s, *v;    /* attacking ship, victim */
    int j;

    if (range < 1) {
        return;
    }

    while (i--) {
        do {
            // fprintf(stderr, "ship %d ", i);
            s = a->active[i];
            // fprintf(stderr, "(type %d): ", s->type);
            j = (int) (rand * range);
            // fprintf(stderr, "target %d ", j);
            v = d->active[j];
            // fprintf(stderr, "(type %d) ", v->type);

            if (ineffective(s, v)) {
                // fprintf(stderr, "ineffective!\n");
                break;
            }

            v->shield -= s->weapon;
            if (v->shield < 0) {
                v->hull += v->shield;
                v->shield = 0;
            }
            // fprintf(stderr, "\n");

        } while (rapidfire_succeeds(s, v));
    }
}


/*
 * explosion
 */

int
explodes(Ship *s)
{
    float hull = s->hull / s->max_hull;
    return hull < .7 ? (rand > hull) : 0;
}

void
explosion(Fleet *f)
{
    Ship **tmp;
    int i = 0, o = 0;
    Ship *s;

    for ( ; i < f->count; ++i) {
	s = f->active[i];
	if (explodes(s)) {
            ++f->lost[s->fleet_id][s->type];
	}
	else {
	    f->alt[o++] = s;
	}
    }

    fprintf(stderr, "surviving ships: %d\n", o);

    tmp = f->alt;
    f->alt = f->active;
    f->active = tmp;
    f->count = o;
}


/*
 * reset shields  for next round
 */

void
reset_shields(Fleet *f)
{
    int i = f->count;
    while (i--) {
	f->active[i]->shield = f->active[i]->max_shield;
    }
}


/*
 * reset fleet  for new battle (with same parameters)
 */

void
reset(Fleet *f)
{
    int j;
    int i = f->start;

    f->count = f->start;

    while (i--) {
	// f->data[i].shield = f->data[i].max_shield;  /* always done */
	f->data[i].hull = f->data[i].max_hull;
        f->active[i] = f->data + i;
    }

    for (i = 0; i < f->fleets; ++i) {
	for (j = 0; j < STZ; ++j) {
	    f->lost[i][j] = 0;
	}
    }
}

/*
 * simulate  - run simulation
 */

int
simulate(Fleet *att, Fleet *def)
{
    int round, asi;

    reset(att);
    reset(def);

    for (round = 0; round < 6; ++round) {
	if (att->count == 0 || def->count == 0)
	    break;
        fprintf(stderr, "battle round: %d\n", round);
        reset_shields(att);
        reset_shields(def);
        fprintf(stderr, "attacker attacks:\n");
        attack(att, def);
        fprintf(stderr, "defender attacks:\n");
        attack(def, att);
        fprintf(stderr, "attacker explosions:\n");
        explosion(att);
        fprintf(stderr, "defender explosions:\n");
        explosion(def);
    }

    asi = att->count == 0 ?
            (def->count == 0 ? 0 : -7 + round) :
            (def->count == 0 ? 7 - round : 0);

    return asi;
}


void
fleet_alloc(Fleet *f, int fleets, int ships)
{
    fprintf(stderr, "fleet alloc %d fleets, %d ships\n", fleets, ships);
    f->start = ships;
    f->fleets = fleets;
    f->active = (Ship **)malloc(ships * sizeof(Ship *));
    f->alt = (Ship **)malloc(ships * sizeof(Ship *));
    f->data = (Ship *)malloc(ships * sizeof(Ship));
    f->lost = (int (*)[STZ])malloc(fleets * sizeof(int (*)[STZ]));
}


void
fleet_free(Fleet *f)
{
    free(f->active);
    free(f->alt);
    free(f->data);
    free(f->lost);
}


void
fleet_init(Fleet *f)
{
    f->count = f->start = 0;
    f->active = NULL;
    f->alt = NULL;
    f->data = NULL;

    f->fleets = 0;
    f->lost = NULL;
}
